import numpy as np

from gpaw.utilities import pack, unpack2
from gpaw.utilities.blas import gemm, axpy
from gpaw.utilities.partition import AtomPartition, AtomicMatrixDistributor
from gpaw.utilities.timing import nulltimer


class EmptyWaveFunctions:
    mode = 'undefined'
    
    def __bool__(self):
        return False

    __nonzero__ = __bool__  # for Python 2
    
    def set_eigensolver(self, eigensolver):
        pass

    def set_orthonormalized(self, flag):
        pass

    def estimate_memory(self, mem):
        mem.set('Unknown WFs', 0)


class WaveFunctions(EmptyWaveFunctions):
    """...

    setups:
        List of setup objects.
    symmetry:
        Symmetry object.
    kpt_u:
        List of **k**-point objects.
    nbands: int
        Number of bands.
    nspins: int
        Number of spins.
    dtype: dtype
        Data type of wave functions (float or complex).
    bzk_kc: ndarray
        Scaled **k**-points used for sampling the whole
        Brillouin zone - values scaled to [-0.5, 0.5).
    ibzk_kc: ndarray
        Scaled **k**-points in the irreducible part of the
        Brillouin zone.
    weight_k: ndarray
        Weights of the **k**-points in the irreducible part
        of the Brillouin zone (summing up to 1).
    kpt_comm:
        MPI-communicator for parallelization over **k**-points.
    """

    collinear = True
    # ncomp is a number of array components necessary to hold
    # information about non-collinear spins.  With collinear spin
    # it is always 1; else it is 2.
    ncomp = 1
    
    def __init__(self, gd, nvalence, setups, bd, dtype,
                 world, kd, kptband_comm, timer):
        self.gd = gd
        self.nspins = kd.nspins
        self.ns = self.nspins * self.ncomp**2
        self.nvalence = nvalence
        self.bd = bd
        #self.nbands = self.bd.nbands #XXX
        #self.mynbands = self.bd.mynbands #XXX
        self.dtype = dtype
        assert dtype == float or dtype == complex
        self.world = world
        self.kd = kd
        self.kptband_comm = kptband_comm
        self.band_comm = self.bd.comm  # XXX
        self.timer = timer
        self.atom_partition = None
        self.big_atom_partition = None
            
        self.kpt_u = kd.create_k_points(self.gd)

        self.eigensolver = None
        self.positions_set = False

        self.amd = None  # matrix distributor.  Switch to better name
        self.set_setups(setups)

    def set_setups(self, setups):
        self.setups = setups

    def set_eigensolver(self, eigensolver):
        self.eigensolver = eigensolver

    def __bool__(self):
        return True

    __nonzero__ = __bool__  # for Python 2

    def add_realspace_orbital_to_density(self, nt_G, psit_G):
        if psit_G.dtype == float:
            axpy(1.0, psit_G**2, nt_G)
        else:
            assert psit_G.dtype == complex
            axpy(1.0, psit_G.real**2, nt_G)
            axpy(1.0, psit_G.imag**2, nt_G)

    def add_orbital_density(self, nt_G, kpt, n):
        self.add_realspace_orbital_to_density(nt_G, kpt.psit_nG[n])

    def calculate_density_contribution(self, nt_sG):
        """Calculate contribution to pseudo density from wave functions.

        Array entries are written to (not added to)."""
        nt_sG.fill(0.0)
        for kpt in self.kpt_u:
            self.add_to_density_from_k_point(nt_sG, kpt)
        self.kptband_comm.sum(nt_sG)
        
        self.timer.start('Symmetrize density')
        for nt_G in nt_sG:
            self.kd.symmetry.symmetrize(nt_G, self.gd)
        self.timer.stop('Symmetrize density')

    def add_to_density_from_k_point(self, nt_sG, kpt):
        self.add_to_density_from_k_point_with_occupation(nt_sG, kpt, kpt.f_n)

    def get_orbital_density_matrix(self, a, kpt, n):
        """Add the nth band density from kpt to density matrix D_sp"""
        ni = self.setups[a].ni
        D_sii = np.zeros((self.nspins, ni, ni))
        P_i = kpt.P_ani[a][n]
        D_sii[kpt.s] += np.outer(P_i.conj(), P_i).real
        D_sp = [pack(D_ii) for D_ii in D_sii]
        return D_sp
    
    def calculate_atomic_density_matrices_k_point(self, D_sii, kpt, a, f_n):
        if kpt.rho_MM is not None:
            P_Mi = self.P_aqMi[a][kpt.q]
            #P_Mi = kpt.P_aMi_sparse[a]
            #ind = get_matrix_index(kpt.P_aMi_index[a])
            #D_sii[kpt.s] += np.dot(np.dot(P_Mi.T.conj(), kpt.rho_MM),
            #                       P_Mi).real
            rhoP_Mi = np.zeros_like(P_Mi)
            D_ii = np.zeros(D_sii[kpt.s].shape, kpt.rho_MM.dtype)
            #gemm(1.0, P_Mi, kpt.rho_MM[ind.T, ind], 0.0, tmp)
            gemm(1.0, P_Mi, kpt.rho_MM, 0.0, rhoP_Mi)
            gemm(1.0, rhoP_Mi, P_Mi.T.conj().copy(), 0.0, D_ii)
            D_sii[kpt.s] += D_ii.real
            #D_sii[kpt.s] += dot(dot(P_Mi.T.conj().copy(),
            #                        kpt.rho_MM[ind.T, ind]), P_Mi).real
        else:
            P_ni = kpt.P_ani[a]
            D_sii[kpt.s] += np.dot(P_ni.T.conj() * f_n, P_ni).real

        if hasattr(kpt, 'c_on'):
            for ne, c_n in zip(kpt.ne_o, kpt.c_on):
                d_nn = ne * np.outer(c_n.conj(), c_n)
                D_sii[kpt.s] += np.dot(P_ni.T.conj(), np.dot(d_nn, P_ni)).real
    
    def calculate_atomic_density_matrices(self, D_asp):
        """Calculate atomic density matrices from projections."""
        f_un = [kpt.f_n for kpt in self.kpt_u]
        self.calculate_atomic_density_matrices_with_occupation(D_asp, f_un)

    def calculate_atomic_density_matrices_with_occupation(self, D_asp, f_un):
        """Calculate atomic density matrices from projections with
        custom occupation f_un."""

        # Parameter check (if user accidentally passes f_n instead of f_un)
        if f_un[0] is not None:  # special case for transport calculations...
            assert isinstance(f_un[0], np.ndarray)
        # Varying f_n used in calculation of response part of GLLB-potential
        for a, D_sp in D_asp.items():
            ni = self.setups[a].ni
            D_sii = np.zeros((len(D_sp), ni, ni))
            for f_n, kpt in zip(f_un, self.kpt_u):
                self.calculate_atomic_density_matrices_k_point(D_sii, kpt, a,
                                                               f_n)
            D_sp[:] = [pack(D_ii) for D_ii in D_sii]
            self.kptband_comm.sum(D_sp)

        self.symmetrize_atomic_density_matrices(D_asp)

    def symmetrize_atomic_density_matrices(self, D_asp):
        if len(self.kd.symmetry.op_scc) == 0:
            return
        
        if hasattr(D_asp, 'redistribute'):
            a_sa = self.kd.symmetry.a_sa
            D_asp.redistribute(self.atom_partition.as_serial())
            for s in range(self.nspins):
                D_aii = [unpack2(D_asp[a][s])
                         for a in range(len(D_asp))]
                for a, D_ii in enumerate(D_aii):
                    setup = self.setups[a]
                    D_asp[a][s] = pack(setup.symmetrize(a, D_aii, a_sa))
            D_asp.redistribute(self.atom_partition)
        else:
            all_D_asp = []
            for a, setup in enumerate(self.setups):
                D_sp = D_asp.get(a)
                if D_sp is None:
                    ni = setup.ni
                    D_sp = np.empty((self.ns, ni * (ni + 1) // 2))
                self.gd.comm.broadcast(D_sp, self.atom_partition.rank_a[a])
                all_D_asp.append(D_sp)
 
            for s in range(self.nspins):
                D_aii = [unpack2(D_sp[s]) for D_sp in all_D_asp]
                for a, D_sp in D_asp.items():
                    setup = self.setups[a]
                    D_sp[s] = pack(setup.symmetrize(a, D_aii,
                                                    self.kd.symmetry.a_sa))

    def set_positions(self, spos_ac, atom_partition=None):
        self.positions_set = False
        #rank_a = self.gd.get_ranks_from_positions(spos_ac)
        #atom_partition = AtomPartition(self.gd.comm, rank_a)
        # XXX pass AtomPartition around instead of spos_ac?
        # All the classes passing around spos_ac end up needing the ranks
        # anyway.

        if atom_partition is None:
            rank_a = self.gd.get_ranks_from_positions(spos_ac)
            atom_partition = AtomPartition(self.gd.comm, rank_a)

        if self.atom_partition is not None and self.kpt_u[0].P_ani is not None:
            self.timer.start('Redistribute')
            mynks = len(self.kpt_u)
            
            def get_empty(a):
                ni = self.setups[a].ni
                return np.empty((mynks, self.bd.mynbands, ni), self.dtype)
            self.atom_partition.redistribute(atom_partition,
                                             [kpt.P_ani for kpt in self.kpt_u],
                                             get_empty)
            self.timer.stop('Redistribute')

        self.atom_partition = atom_partition
        self.kd.symmetry.check(spos_ac)

    def allocate_arrays_for_projections(self, my_atom_indices):
        if not self.positions_set and self.kpt_u[0].P_ani is not None:
            # Projections have been read from file - don't delete them!
            pass
        else:
            for kpt in self.kpt_u:
                kpt.P_ani = {}
            for a in my_atom_indices:
                ni = self.setups[a].ni
                for kpt in self.kpt_u:
                    kpt.P_ani[a] = np.empty((self.bd.mynbands, ni), self.dtype)

    def collect_eigenvalues(self, k, s):
        return self.collect_array('eps_n', k, s)
    
    def collect_occupations(self, k, s):
        return self.collect_array('f_n', k, s)

    def collect_array(self, name, k, s, subset=None):
        """Helper method for collect_eigenvalues and collect_occupations.

        For the parallel case find the rank in kpt_comm that contains
        the (k,s) pair, for this rank, collect on the corresponding
        domain a full array on the domain master and send this to the
        global master."""

        kpt_u = self.kpt_u
        kpt_rank, u = self.kd.get_rank_and_index(s, k)

        if self.kd.comm.rank == kpt_rank:
            a_nx = getattr(kpt_u[u], name)

            if subset is not None:
                a_nx = a_nx[subset]

            # Domain master send this to the global master
            if self.gd.comm.rank == 0:
                if self.band_comm.size == 1:
                    if kpt_rank == 0:
                        return a_nx
                    else:
                        self.kd.comm.ssend(a_nx, 0, 1301)
                else:
                    b_nx = self.bd.collect(a_nx)
                    if self.band_comm.rank == 0:
                        if kpt_rank == 0:
                            return b_nx
                        else:
                            self.kd.comm.ssend(b_nx, 0, 1301)

        elif self.world.rank == 0 and kpt_rank != 0:
            # Only used to determine shape and dtype of receiving buffer:
            a_nx = getattr(kpt_u[0], name)

            if subset is not None:
                a_nx = a_nx[subset]

            b_nx = np.zeros((self.bd.nbands,) + a_nx.shape[1:],
                            dtype=a_nx.dtype)
            self.kd.comm.receive(b_nx, kpt_rank, 1301)
            return b_nx

    def collect_auxiliary(self, value, k, s, shape=1, dtype=float):
        """Helper method for collecting band-independent scalars/arrays.

        For the parallel case find the rank in kpt_comm that contains
        the (k,s) pair, for this rank, collect on the corresponding
        domain a full array on the domain master and send this to the
        global master."""

        kpt_u = self.kpt_u
        kpt_rank, u = self.kd.get_rank_and_index(s, k)

        if self.kd.comm.rank == kpt_rank:
            if isinstance(value, str):
                a_o = getattr(kpt_u[u], value)
            else:
                a_o = value[u]  # assumed list

            # Make sure data is a mutable object
            a_o = np.asarray(a_o)

            if a_o.dtype is not dtype:
                a_o = a_o.astype(dtype)

            # Domain master send this to the global master
            if self.gd.comm.rank == 0:
                if kpt_rank == 0:
                    return a_o
                else:
                    self.kd.comm.send(a_o, 0, 1302)

        elif self.world.rank == 0 and kpt_rank != 0:
            b_o = np.zeros(shape, dtype=dtype)
            self.kd.comm.receive(b_o, kpt_rank, 1302)
            return b_o

    def collect_projections(self, k, s):
        """Helper method for collecting projector overlaps across domains.

        For the parallel case find the rank in kpt_comm that contains
        the (k,s) pair, for this rank, send to the global master."""

        kpt_rank, u = self.kd.get_rank_and_index(s, k)

        natoms = self.atom_partition.natoms
        nproj = sum([setup.ni for setup in self.setups])

        if self.world.rank == 0:
            if kpt_rank == 0:
                P_ani = self.kpt_u[u].P_ani
            all_P_ni = np.empty((self.bd.nbands, nproj), self.dtype)
            for band_rank in range(self.band_comm.size):
                nslice = self.bd.get_slice(band_rank)
                i = 0
                for a in range(natoms):
                    ni = self.setups[a].ni
                    if kpt_rank == 0 and band_rank == 0 and a in P_ani:
                        P_ni = P_ani[a]
                    else:
                        P_ni = np.empty((self.bd.mynbands, ni), self.dtype)
                        # XXX will fail with nonstandard communicator nesting
                        world_rank = (self.atom_partition.rank_a[a] +
                                      kpt_rank * self.gd.comm.size *
                                      self.band_comm.size +
                                      band_rank * self.gd.comm.size)
                        self.world.receive(P_ni, world_rank, 1303 + a)
                    all_P_ni[nslice, i:i + ni] = P_ni
                    i += ni
                assert i == nproj
            return all_P_ni

        elif self.kd.comm.rank == kpt_rank:  # plain else works too...
            P_ani = self.kpt_u[u].P_ani
            for a in range(natoms):
                if a in P_ani:
                    self.world.ssend(P_ani[a], 0, 1303 + a)

    def get_wave_function_array(self, n, k, s, realspace=True):
        """Return pseudo-wave-function array on master.
        
        n: int
            Global band index.
        k: int
            Global IBZ k-point index.
        s: int
            Spin index (0 or 1).
        realspace: bool
            Transform plane wave or LCAO expansion coefficients to real-space.

        For the parallel case find the ranks in kd.comm and bd.comm
        that contains to (n, k, s), and collect on the corresponding
        domain a full array on the domain master and send this to the
        global master."""

        kpt_rank, u = self.kd.get_rank_and_index(s, k)
        band_rank, myn = self.bd.who_has(n)

        rank = self.world.rank

        if (self.kd.comm.rank == kpt_rank and
            self.band_comm.rank == band_rank):
            psit_G = self._get_wave_function_array(u, myn, realspace)
            if realspace:
                psit_G = self.gd.collect(psit_G)

            if rank == 0:
                return psit_G
            
            # Domain master send this to the global master
            if self.gd.comm.rank == 0:
                self.world.ssend(psit_G, 0, 1398)

        if rank == 0:
            # allocate full wave function and receive
            psit_G = self.empty(global_array=True,
                                realspace=realspace)
            # XXX this will fail when using non-standard nesting
            # of communicators.
            world_rank = (kpt_rank * self.gd.comm.size *
                          self.band_comm.size +
                          band_rank * self.gd.comm.size)
            self.world.receive(psit_G, world_rank, 1398)
            return psit_G
            
    def read_projections(self, reader):
        nslice = self.bd.get_slice()
        for u, kpt in enumerate(self.kpt_u):
            P_nI = reader.get('Projections', kpt.s, kpt.k)
            I1 = 0
            kpt.P_ani = {}
            for a, setup in enumerate(self.setups):
                I2 = I1 + setup.ni
                if self.gd.comm.rank == 0:
                    kpt.P_ani[a] = np.array(P_nI[nslice, I1:I2], self.dtype)
                I1 = I2



import numpy as np
from scipy.special import dawsn

from gpaw.gauss import Gauss, Lorentz


class ComplexLorentz:
    def __init__(self, width=0.08):
        self.dtype = complex
        self.set_width(width)
        
    def get(self, x, x0):
        return 0.5 / x0 * (((x + x0) / ((x + x0)**2 + self.width2)
                            - (x - x0) / ((x - x0)**2 + self.width2))
                           - 1.0j * self.width *
                           (1 / ((x + x0)**2 + self.width2)
                            - 1 / ((x - x0)**2 + self.width2))
                           )
        
    def set_width(self, width=0.08):
        self.width = width
        self.width2 = width**2


class ComplexGauss:
    def __init__(self, width=0.08):
        self.dtype = complex
        self.set_width(width)
        
    def get(self, x, x0):
        return 0.5 / x0 * (2 * self.wm1 *
                           (dawsn((x + x0) * self.wm1)
                            - dawsn((x - x0) * self.wm1))
                           - 1.0j * self.norm *
                           (np.exp(-((x + x0) * self.wm1)**2)
                            - np.exp(-((x - x0) * self.wm1)**2))
                           )
    
    def set_width(self, width=0.08):
        self.norm = 1. / width * np.sqrt(np.pi / 2)
        self.wm1 = np.sqrt(.5) / width


class LorentzPole:
    """Pole of a damped harmonic oscillator.

    See: e.g. Frederick Wooten, Optical properties of solids,
    Academic Press Inc. (1972)"""
    def __init__(self, width=0.08, imag=True):
        self.dtype = float
        if imag:
            self.get = self.get_imaginary
        else:
            self.get = self.get_real
        self.set_width(width)
        
    def get_real(self, x, x0):
        return (x0**2 - x**2) / (
            (x0**2 - x**2)**2 + self.width**2 * x**2)
    
    def get_imaginary(self, x, x0):
        return self.width * x / (
            (x0**2 - x**2)**2 + self.width**2 * x**2)
    
    def set_width(self, width=0.08):
        self.width = width
    

class Folder:
    """Fold a function with normalised Gaussians or Lorentzians

    Example: fold a function y(x) by Lorentzians of 0.2 width

    >>> xlist = ... # list of x-values
    >>> ylist = ... # corresponding list of y(x)-values

    >>> from gpaw.utilities.folder import Folder
    >>> fxlist, fylist = Folder(width=0.2, folding='Lorentz').fold(
                     xlist, ylist, dx=0.1, xmin=-1, xmax=1)
"""
    def __init__(self, width,
                 folding='Gauss'):
        self.width = width
        if folding == 'Gauss':
            self.func = Gauss(width)
        elif folding == 'Lorentz':
            self.func = Lorentz(width)
        elif folding == 'ComplexGauss':
            self.func = ComplexGauss(width)
        elif folding == 'ComplexLorentz':
            self.func = ComplexLorentz(width)
        elif folding == 'RealLorentzPole':
            self.func = LorentzPole(width, imag=False)
        elif folding == 'ImaginaryLorentzPole':
            self.func = LorentzPole(width, imag=True)
        elif folding is None:
            self.func = None
        else:
            raise RuntimeError('unknown folding "' + folding + '"')

    def fold(self, x, y, dx=None, xmin=None, xmax=None):
        X = np.array(x)
        assert len(X.shape) == 1
        Y = np.array(y)
        assert X.shape[0] == Y.shape[0]

        if self.func is None:
            xl = X
            yl = Y
        else:
            if xmin is None:
                xmin = np.min(X) - 4 * self.width
            if xmax is None:
                xmax = np.max(X) + 4 * self.width
            if dx is None:
                dx = self.width / 4.

            xl = np.arange(xmin, xmax + 0.5 * dx, dx)
            
            xl, yl = self.fold_values(x, y, xl)
            
        return xl, yl

    def fold_values(self, x, y, xl=None):
        X = np.array(x)
        assert len(X.shape) == 1
        Y = np.array(y)
        assert X.shape[0] == Y.shape[0]

        if self.func is None:
            xl = X
            yl = Y
        else:
            if xl is None:
                Xl = np.unique(X)
            else:
                Xl = np.array(xl)
                assert len(Xl.shape) == 1
            
            # weight matrix
            weightm = np.empty((Xl.shape[0], X.shape[0]),
                               dtype=self.func.dtype)
            for i, x in enumerate(X):
                weightm[:, i] = self.func.get(Xl, x)

#            yl = np.dot(weightm, Y)
            yl = np.tensordot(weightm, Y, axes=(1, 0))

        return Xl, yl



from ase import Atoms
from gpaw import GPAW
from gpaw.lrtddft import LrTDDFT
from gpaw.inducedfield.inducedfield_lrtddft import LrTDDFTInducedField
import numpy as np

# 1) Ground state calculation with empty states
atoms = Atoms(symbols='Na2',
              positions=[(0, 0, 0), (3.0, 0, 0)],
              pbc=False)
atoms.center(vacuum=3.0)

calc = GPAW(nbands=20, h=0.6, setups={'Na': '1'})
atoms.set_calculator(calc)
energy = atoms.get_potential_energy()
calc.write('na2_gs_casida.gpw', mode='all')

# 2) Casida calculation
calc = GPAW('na2_gs_casida.gpw')
istart = 0
jend = 20
lr = LrTDDFT(calc, xc='LDA', istart=istart, jend=jend)
lr.diagonalize()
lr.write('na2_lr.dat.gz')

# 3) Calculate induced field
frequencies = [1.0, 2.08]  # Frequencies of interest in eV
folding = 'Gauss'          # Folding function
width = 0.1                # Line width for folding in eV
kickdir = 0                # Kick field direction 0, 1, 2 for x, y, z
ind = LrTDDFTInducedField(paw=calc, lr=lr, frequencies=frequencies,
                          folding=folding, width=width, kickdir=kickdir)
ind.calculate_induced_field(gridrefinement=2, from_density='comp',
                            poisson_eps=2e-10)

# Test
from gpaw.test import equal
tol = 0.0001
val1 = ind.fieldgd.integrate(ind.Ffe_wg[0])
val2 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[0][0]))
val3 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[0][1]))
val4 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[0][2]))
val5 = ind.fieldgd.integrate(ind.Ffe_wg[1])
val6 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[1][0]))
val7 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[1][1]))
val8 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[1][2]))
equal(val1, 3175.76177761, tol)
equal(val2, 1700.43442519, tol)
equal(val3, 1187.26249225, tol)
equal(val4, 1187.26249225, tol)
equal(val5, 10956.9813196, tol)
equal(val6, 6574.58868754, tol)
equal(val7, 4589.74440108, tol)
equal(val8, 4589.74440108, tol)



from math import sqrt, pi

import numpy as np

from gpaw.xc.functional import XCFunctional
from gpaw.xc.lda import LDA
from gpaw.xc.libxc import LibXC
from gpaw.lcao.eigensolver import DirectLCAO
from gpaw.wavefunctions.lcao import LCAOWaveFunctions
from gpaw.utilities import unpack
from gpaw.utilities.blas import gemm
from gpaw.mixer import BaseMixer
from gpaw.utilities.tools import tri2full
from gpaw.sphere.lebedev import Y_nL, weight_n
from gpaw.xc.pawcorrection import rnablaY_nLv

X = np.newaxis


class NonCollinearLDAKernel(LibXC):
    def __init__(self):
        LibXC.__init__(self, 'LDA')
        
    def calculate(self, e_g, n_sg, dedn_sg,
                  sigma_xg=None, dedsigma_xg=None,
                  tau_sg=None, dedtau_sg=None):
        n_g = n_sg[0]
        m_vg = n_sg[1:4]
        m_g = (m_vg**2).sum(0)**0.5
        nnew_sg = np.empty((2,) + n_g.shape)
        nnew_sg[:] = n_g
        nnew_sg[0] += m_g
        nnew_sg[1] -= m_g
        nnew_sg *= 0.5
        vnew_sg = np.zeros_like(nnew_sg)
        LibXC.calculate(self, e_g, nnew_sg, vnew_sg)
        dedn_sg[0] += 0.5 * vnew_sg.sum(0)
        vnew_sg /= np.where(m_g < 1e-15, 1, m_g)
        dedn_sg[1:4] += 0.5 * vnew_sg[0] * m_vg
        dedn_sg[1:4] -= 0.5 * vnew_sg[1] * m_vg


class NonCollinearFunctional(XCFunctional):
    def __init__(self, xc):
        XCFunctional.__init__(self, xc.name)
        self.xc = xc
        self.type = xc.type
        
    def calculate(self, gd, n_sg, dedn_sg=None, e_g=None):
        n_g = n_sg[0]
        m_vg = n_sg[1:4]
        m_g = (m_vg**2).sum(0)**0.5
        nnew_sg = gd.empty(2)
        nnew_sg[:] = n_g
        nnew_sg[0] += m_g
        nnew_sg[1] -= m_g
        nnew_sg *= 0.5
        vnew_sg = gd.zeros(2)
        if e_g is None:
            e_g = gd.empty()
        exc = self.xc.calculate(gd, nnew_sg, vnew_sg, e_g)
        if dedn_sg is not None:
            dedn_sg[0] += 0.5 * vnew_sg.sum(0)
            vnew_sg /= np.where(m_g < 1e-15, 1, m_g)
            dedn_sg[1:4] += 0.5 * vnew_sg[0] * m_vg
            dedn_sg[1:4] -= 0.5 * vnew_sg[1] * m_vg
        return exc
    
    def calculate_paw_correction(self, setup, D_sp, dEdD_sp=None,
                                 addcoredensity=True, a=None):
        c = setup.xc_correction
        if c is None:
            return 0.0
        
        assert addcoredensity
        assert len(D_sp) == 4
        assert c.nc_corehole_g is None
        
        rgd = c.rgd
        
        nc0_g = sqrt(4 * pi) * c.nc_g
        nct0_g = sqrt(4 * pi) * c.nct_g
        
        D_sLq = np.inner(D_sp, c.B_pqL.T)

        e, dEdD_sqL = self.calculate_radial_expansion(rgd, D_sLq, c.n_qg,
                                                      nc0_g)
        et, dEtdD_sqL = self.calculate_radial_expansion(rgd, D_sLq, c.nt_qg,
                                                        nct0_g)

        if dEdD_sp is not None:
            dEdD_sp += np.inner((dEdD_sqL - dEtdD_sqL).reshape((4, -1)),
                                c.B_pqL.reshape((len(c.B_pqL), -1)))

        return e - et - c.Exc0

    def calculate_radial_expansion(self, rgd, D_sLq, n_qg, nc0_g):
        D_Lq = D_sLq[0]
        M_vLq = D_sLq[1:]
        
        n_Lg = np.dot(D_Lq, n_qg)
        n_Lg[0] += nc0_g

        m_vLg = np.dot(M_vLq, n_qg)

        if self.xc.type == 'GGA':
            dndr_Lg = np.empty_like(n_Lg)
            rgd.derivative(n_Lg, dndr_Lg)
            dmdr_vLg = np.empty_like(m_vLg)
            rgd.derivative(m_vLg, dmdr_vLg)
        elif self.xc.type == 'LDA':
            dndr_Lg = None
            dmdr_vLg = None

        Lmax, nq = D_Lq.shape
        dEdD_sqL = np.zeros((4, nq, Lmax))
        
        E = 0.0
        for w, Y_L, rnablaY_Lv in zip(weight_n,
                                      Y_nL[:, :Lmax],
                                      rnablaY_nLv[:, :Lmax]):
            if self.xc.type == 'LDA':
                (e_g, dedn_g, dedm_vg) = \
                    self.calculate_radial(rgd, n_Lg, Y_L, dndr_Lg, rnablaY_Lv,
                                          m_vLg, dmdr_vLg)
            else:
                (e_g, dedn_g, dedm_vg,
                 a_g, b_vg, c_g, d_vg, dedsigma_xg, dcdm_vg, dddm_vLvg,
                 m_vg, m_g) = \
                 self.calculate_radial(rgd, n_Lg, Y_L, dndr_Lg, rnablaY_Lv,
                                       m_vLg, dmdr_vLg)
            dEdD_sqL[0] += np.dot(rgd.dv_g * dedn_g,
                                  n_qg.T)[:, X] * (w * Y_L)
            dEdD_sqL[1:] += np.dot(rgd.dv_g * dedm_vg,
                                   n_qg.T)[:, :, X] * (w * Y_L)
            if self.xc.type == 'GGA':
                v_g = rgd.empty()
                rgd.derivative2(rgd.dv_g * (dedsigma_xg[0] * (a_g + c_g) +
                                            dedsigma_xg[1] * a_g +
                                            dedsigma_xg[2] * (a_g - c_g)), v_g)
                dEdD_sqL[0] -= np.dot(v_g, n_qg.T)[:, X] * (0.5 * w * Y_L)
                B_vg = (dedsigma_xg[0] * (b_vg + d_vg) +
                        dedsigma_xg[1] * b_vg +
                        dedsigma_xg[2] * (b_vg - d_vg))
                B_vq = np.dot(B_vg * rgd.dr_g, n_qg.T)
                dEdD_sqL[0] += 2 * pi * w * np.dot(rnablaY_Lv, B_vq).T

                A_g = (dedsigma_xg[0] * (a_g + c_g) -
                       dedsigma_xg[1] * c_g +
                       dedsigma_xg[2] * (c_g - a_g)) * rgd.dv_g
                v_vg = rgd.empty(3)
                rgd.derivative2(A_g * m_vg / m_g, v_vg)
                v_vg -= A_g * dcdm_vg
                dEdD_sqL[1:] -= np.dot(v_vg, n_qg.T)[:, :, X] * (0.5 * w * Y_L)

                dedsigma_xg *= rgd.dr_g
                B_vg = (dedsigma_xg[0] * (b_vg + d_vg) -
                        dedsigma_xg[1] * d_vg +
                        dedsigma_xg[2] * (d_vg - b_vg))
                B_Lvq = np.dot((B_vg[:, X, X] * dddm_vLvg).sum(0), n_qg.T)
                dEdD_sqL[1:] += 2 * pi * w * B_Lvq.transpose((1, 2, 0))
                
            E += w * rgd.integrate(e_g)

        return E, dEdD_sqL

    def calculate_radial(self, rgd, n_Lg, Y_L, dndr_Lg, rnablaY_Lv,
                         m_vLg, dmdr_vLg):
        n_g = np.dot(Y_L, n_Lg)
        m_vg = np.dot(Y_L, m_vLg)
        m_g = (m_vg**2).sum(0)**0.5
        eps = 1e-15
        m_g[m_g < eps] = eps
        n_sg = rgd.empty(2)
        n_sg[:] = n_g
        n_sg[0] += m_g
        n_sg[1] -= m_g
        n_sg *= 0.5

        e_g = rgd.empty()
        dedn_sg = rgd.zeros(2)

        if self.xc.type == 'GGA':
            dmdr_vg = np.dot(Y_L, dmdr_vLg)
        
            a_g = np.dot(Y_L, dndr_Lg)
            b_vg = np.dot(rnablaY_Lv.T, n_Lg)
            
            c_g = (m_vg * dmdr_vg).sum(0) / m_g
            m_vvg = np.dot(rnablaY_Lv.T, m_vLg)
            d_vg = (m_vg * m_vvg).sum(1) / m_g

            sigma_xg = rgd.empty(3)
            sigma_xg[0] = ((b_vg + d_vg)**2).sum(0)
            sigma_xg[1] = ((b_vg + d_vg) * (b_vg - d_vg)).sum(0)
            sigma_xg[2] = ((b_vg - d_vg)**2).sum(0)
            sigma_xg[:, 1:] /= rgd.r_g[1:]**2
            sigma_xg[:, 0] = sigma_xg[:, 1]
            sigma_xg[0] += (a_g + c_g)**2
            sigma_xg[1] += (a_g + c_g) * (a_g - c_g)
            sigma_xg[2] += (a_g - c_g)**2
            sigma_xg *= 0.25

            dedsigma_xg = rgd.zeros(3)

            self.xc.kernel.calculate(e_g, n_sg, dedn_sg, sigma_xg, dedsigma_xg)
        else:
            self.xc.kernel.calculate(e_g, n_sg, dedn_sg)

        dedn_g = 0.5 * dedn_sg.sum(0)
        dedn_sg /= m_g
        dedm_vg = 0.5 * (dedn_sg[0] - dedn_sg[1]) * m_vg
        if self.xc.type == 'GGA':
            dcdm_vg = (dmdr_vg - (m_vg * dmdr_vg).sum(0) * m_vg / m_g**2) / m_g
            dddm_vLvg = rnablaY_Lv.T[:, :, X, X] * m_vg
            dddm_vLvg += m_vvg[:, X] * Y_L[:, X, X]
            dddm_vLvg -= d_vg[:, X, X, :] * m_vg[X, X] * Y_L[:, X, X] / m_g
            dddm_vLvg /= m_g

            return (e_g, dedn_g, dedm_vg,
                    a_g, b_vg, c_g, d_vg, dedsigma_xg, dcdm_vg, dddm_vLvg,
                    m_vg, m_g)

        return e_g, dedn_g, dedm_vg


class NonCollinearLCAOEigensolver(DirectLCAO):
    def calculate_hamiltonian_matrix(self, ham, wfs, kpt, root=-1):

        assert self.has_initialized

        vt_sG = ham.vt_sG
        dH_asp = ham.dH_asp
        H_MM = np.empty((wfs.ksl.mynao, wfs.ksl.nao), wfs.dtype)
        H_sMsM = np.empty((2, wfs.ksl.mynao, 2, wfs.ksl.nao), complex)
        
        wfs.timer.start('Potential matrix')
        self.get_component(wfs, 0, vt_sG, dH_asp, kpt, H_MM)
        H_sMsM[0, :, 0] = H_MM
        H_sMsM[1, :, 1] = H_MM
        self.get_component(wfs, 1, vt_sG, dH_asp, kpt, H_MM)
        H_sMsM[0, :, 1] = H_MM
        H_sMsM[1, :, 0] = H_MM.conj().T
        self.get_component(wfs, 2, vt_sG, dH_asp, kpt, H_MM)
        H_sMsM[0, :, 1] += 1j * H_MM
        H_sMsM[1, :, 0] -= 1j * H_MM.conj().T
        self.get_component(wfs, 3, vt_sG, dH_asp, kpt, H_MM)
        H_sMsM[0, :, 0] += H_MM
        H_sMsM[1, :, 1] -= H_MM
        wfs.timer.stop('Potential matrix')

        ham.gd.comm.sum(H_sMsM)

        H_sMsM[0, :, 0] += wfs.T_qMM[kpt.q]
        H_sMsM[1, :, 1] += wfs.T_qMM[kpt.q]

        wfs.timer.start('Distribute overlap matrix')
        #H_MM = wfs.ksl.distribute_overlap_matrix(H_MM, root)
        wfs.timer.stop('Distribute overlap matrix')
        H_sMsM.shape = (2 * wfs.ksl.mynao, 2 * wfs.ksl.nao)
        return H_sMsM

    def get_component(self, wfs, s, vt_sG, dH_asp, kpt, H_MM):
        wfs.basis_functions.calculate_potential_matrix(vt_sG[s], H_MM, kpt.q)

        # Add atomic contribution
        #
        #           --   a     a  a*
        # H      += >   P    dH  P
        #  mu nu    --   mu i  ij nu j
        #           aij
        #
        wfs.timer.start('Atomic Hamiltonian')
        Mstart = wfs.basis_functions.Mstart
        Mstop = wfs.basis_functions.Mstop
        wfs.timer.stop('Atomic Hamiltonian')
        tri2full(H_MM)
        for a, P_qMi in wfs.P_aqMi.items():
            P_Mi = P_qMi[kpt.q]
            dH_ii = np.asarray(unpack(dH_asp[a][s]), wfs.dtype)
            dHP_iM = np.zeros((dH_ii.shape[1], P_Mi.shape[0]), wfs.dtype)
            # (ATLAS can't handle uninitialized output array)
            gemm(1.0, P_Mi, dH_ii, 0.0, dHP_iM, 'c')
            gemm(1.0, dHP_iM, P_Mi[Mstart:Mstop], 1.0, H_MM)

    def iterate(self, hamiltonian, wfs):
        wfs.timer.start('Noncollinear LCAO eigensolver')

        for kpt in wfs.kpt_u:
            self.iterate_one_k_point(hamiltonian, wfs, kpt)

        wfs.timer.stop('Noncollinear LCAO eigensolver')

    def iterate_one_k_point(self, hamiltonian, wfs, kpt):
        if wfs.bd.comm.size > 1 and wfs.bd.strided:
            raise NotImplementedError

        H_MM = self.calculate_hamiltonian_matrix(hamiltonian, wfs, kpt, root=0)
        S_MM = np.zeros_like(H_MM)
        nao = wfs.ksl.nao
        S_MM.shape = (2, nao, 2, nao)
        for s in range(2):
            S_MM[s, :, s] = wfs.S_qMM[kpt.q]
        S_MM.shape = (2 * nao, 2 * nao)

        if kpt.eps_n is None:
            kpt.eps_n = np.empty(wfs.bd.mynbands)

        diagonalization_string = repr(self.diagonalizer)
        wfs.timer.start(diagonalization_string)
        kpt.C_nsM.shape = (wfs.bd.mynbands, 2 * nao)
        self.diagonalizer.diagonalize(H_MM, kpt.C_nsM, kpt.eps_n, S_MM)
        kpt.C_nsM.shape = (wfs.bd.mynbands, 2, nao)
        wfs.timer.stop(diagonalization_string)


class NonCollinearLCAOWaveFunctions(LCAOWaveFunctions):
    collinear = False
    ncomp = 2

    def set_positions(self, spos_ac, atom_partition):
        LCAOWaveFunctions.set_positions(self, spos_ac, atom_partition)
        for kpt in self.kpt_u:
            kpt.C_nM = None
            kpt.C_nsM = np.empty((self.bd.mynbands, 2, self.ksl.nao), complex)
            
    def add_to_density_from_k_point_with_occupation(self, nt_sG, kpt, f_n):
        rho00_MM = self.ksl.calculate_density_matrix(f_n,
                                                     kpt.C_nsM[:, 0].copy())
        rho01_MM = self.ksl.calculate_density_matrix(
            f_n,
            kpt.C_nsM[:, 0].copy(),
            C2_nM=kpt.C_nsM[:, 1].copy())
        rho11_MM = self.ksl.calculate_density_matrix(f_n,
                                                     kpt.C_nsM[:, 1].copy())
        kpt.rho_sMM = np.empty((4, self.ksl.nao, self.ksl.nao), self.dtype)
        if self.dtype == float:
            kpt.rho_sMM[0] = rho00_MM.real + rho11_MM.real
            kpt.rho_sMM[1] = rho01_MM.real + rho01_MM.T.real
            kpt.rho_sMM[2] = -rho01_MM.imag - rho01_MM.T.imag
            kpt.rho_sMM[3] = rho00_MM.real - rho11_MM.real
        else:
            kpt.rho_sMM[0] = rho00_MM + rho11_MM
            kpt.rho_sMM[1] = rho01_MM + rho01_MM.T.conj()
            kpt.rho_sMM[2] = 1j * (rho01_MM - rho01_MM.T.conj())
            kpt.rho_sMM[3] = rho00_MM - rho11_MM
        for rho_MM, nt_G in zip(kpt.rho_sMM, nt_sG):
            self.basis_functions.construct_density(rho_MM, nt_G, kpt.q)

    def calculate_atomic_density_matrices_k_point(self, D_sii, kpt, a, f_n):
        P_Mi = self.P_aqMi[a][kpt.q]
        rhoP_Mi = np.zeros_like(P_Mi)
        D0_ii = np.zeros(D_sii[0].shape, self.dtype)
        for rho_MM, D_ii in zip(kpt.rho_sMM, D_sii):
            gemm(1.0, P_Mi, rho_MM, 0.0, rhoP_Mi)
            gemm(1.0, rhoP_Mi, P_Mi.T.conj().copy(), 0.0, D0_ii)
            D_ii += 0.5 * (D0_ii.real + D0_ii.T.real)


class NonCollinearMixer(BaseMixer):
    def mix(self, density):
        BaseMixer.mix(self, density.nt_sG[0],
                      [D_sp[0] for D_sp in density.D_asp.values()])



from __future__ import print_function
from time import time
import sys
from fractions import gcd
import numpy as np
from gpaw import parsize_domain, parsize_bands
from gpaw.band_descriptor import BandDescriptor
from gpaw.grid_descriptor import GridDescriptor
from gpaw.kohnsham_layouts import BandLayouts
from gpaw.mpi import world, distribute_cpus
from gpaw.hs_operators import MatrixOperator
from gpaw.fd_operators import Laplace

G = 120  # number of grid points (G x G x G)
N = 2000  # number of bands
repeats = 1

try:
    N = int(sys.argv[1])
    J = int(sys.argv[2])
except (IndexError, ValueError):
    N = 6
    J = 3
    repeats = 3

# B: number of band groups
# D: number of domains
if parsize_bands is None:
    if parsize_domain is None:
        B = gcd(N, world.size)
        D = world.size // B
    else:
        B = world.size // np.prod(parsize_domain)
        D = parsize_domain
else:
    B = parsize_bands
    D = world.size // B

M = N // B     # number of bands per group
assert M * B == N, 'M=%d, B=%d, N=%d' % (M, B, N)

h = 0.2        # grid spacing
a = h * G      # side length of box
assert np.prod(D) * B == world.size, 'D=%s, B=%d, W=%d' % (D, B, world.size)

# Set up communicators:
comms = distribute_cpus(parsize_domain=D,
                        parsize_bands=B,
                        nspins=1, nibzkpts=1)
domain_comm, kpt_comm, band_comm, block_comm = \
    [comms[name] for name in ['d', 'k', 'b', 'K']]

assert kpt_comm.size == 1
if world.rank == 0:
    print('MPI: %d domains, %d band groups' % (domain_comm.size,
                                               band_comm.size))

# Set up band and grid descriptors:
bd = BandDescriptor(N, band_comm, False)
gd = GridDescriptor((G, G, G), (a, a, a), True, domain_comm, parsize_c=D)
ksl = BandLayouts(gd, bd, block_comm, float)

# Random wave functions:
psit_mG = gd.empty(M)
for m in range(M):
    np.random.seed(world.rank * M + m)
    psit_mG[m] = np.random.uniform(-0.5, 0.5, tuple(gd.n_c))
if world.rank == 0:
    print('Size of wave function array:', psit_mG.shape)
P_ani = {0: psit_mG[:, :2, 0, 0].copy(),
         1: psit_mG[:, -1, -1, -3:].copy()}

kin = Laplace(gd, -0.5, 2).apply
vt_G = gd.empty()
vt_G.fill(0.567)


def run(psit_mG):
    overlap = MatrixOperator(ksl, J)

    def H(psit_xG):
        Htpsit_xG = np.empty_like(psit_xG)
        kin(psit_xG, Htpsit_xG)
        for psit_G, y_G in zip(psit_xG, Htpsit_xG):
            y_G += vt_G * psit_G
        return Htpsit_xG
    
    dH_aii = {0: np.ones((2, 2)) * 0.123, 1: np.ones((3, 3)) * 0.321}
    
    def dH(a, P_ni):
        return np.dot(P_ni, dH_aii[a])
    
    H_nn = overlap.calculate_matrix_elements(psit_mG, P_ani, H, dH)

    t1 = time()
    if world.rank == 0:
        eps_n, H_nn = np.linalg.eigh(H_nn)
        H_nn = np.ascontiguousarray(H_nn.T)
    t2 = time()

    if world.rank == 0:
        print('Diagonalization Time %f' % (t2 - t1))
        print(eps_n)

    # Distribute matrix:
    world.broadcast(H_nn, 0)

    psit_mG = overlap.matrix_multiply(H_nn, psit_mG, P_ani)

    if world.rank == 0:
        print('Made it past matrix multiply')

    # Check:
    assert not(P_ani[0] - psit_mG[:, :2, 0, 0]).round(10).any()
    assert not(P_ani[1] - psit_mG[:, -1, -1, -3:]).round(10).any()

    H_nn = overlap.calculate_matrix_elements(psit_mG, P_ani, H, dH)
    if world.rank == 0:
        for n in range(N):
            assert abs(H_nn[n, n] - eps_n[n]) < 2e-8
            assert not H_nn[n + 1:, n].round(8).any()

    return psit_mG

ta = time()

for x in range(repeats):
    psit_mG = run(psit_mG)

tb = time()

if world.rank == 0:
    print('Total Time %f' % (tb - ta))



from ase import Atoms
from gpaw import GPAW

atoms = Atoms('H2', [(0.0, 0.0, 0.0), 
                     (0.0, 0.0, 3.995)], 
              cell=(4, 4, 4), pbc=True)

calc = GPAW(txt=None)
atoms.set_calculator(calc)
calc.initialize(atoms)

try:
    calc.set_positions(atoms)
except RuntimeError:
    pass
else:
    assert 2 + 2 == 5



from __future__ import print_function

"""Test reading of gpaw output by ase"""

from ase.structure import molecule
from ase.io import read

from gpaw import GPAW
from gpaw.cluster import Cluster
from gpaw.occupations import FixedOccupations, ZeroKelvin

calculate = 1

if calculate:
    H2 = Cluster(molecule('H2'))
    h=0.4
    H2.minimal_box(2, 0.4)

    c = GPAW(h=h, nbands=2)
    H2.set_calculator(c)

    c.set(txt='no_spin_no_pbc.out')
    H2.get_potential_energy()

    c.set(spinpol=True, txt='spin_no_pbc.out')
    H2.get_potential_energy()

    H2.set_pbc(True)
    c.set(symmetry='off', kpts=(2,3,1), txt='spin_pbc.out')
    H2.get_potential_energy()

    c.set(spinpol=False, txt='no_spin_pbc.out')
    H2.get_potential_energy()

s = read('no_spin_no_pbc.out')
c = s.get_calculator()
assert(len(c.kpts) == 1)

s = read('spin_no_pbc.out')
c = s.get_calculator()
assert(len(c.kpts) == 2)

s = read('no_spin_pbc.out')
c = s.get_calculator()
assert(len(c.kpts) == 6)

s = read('spin_pbc.out')
c = s.get_calculator()
assert(len(c.kpts) == 12)




# -*- coding: utf-8 -*-
# Copyright (C) 2008  CSC Scientific Computing Ltd.
# Please see the accompanying LICENSE file for further information.

"""This module defines an Overlap operator.

The module defines an overlap operator and implements overlap-related
functions.

"""
import numpy as np

from gpaw.utilities.tools import tri2full
from gpaw import extra_parameters
from gpaw.utilities.lapack import diagonalize

class Overlap:
    """Overlap operator S

    This class contains information required to apply the
    overlap operator to a set of wavefunctions.
    """

    def __init__(self, ksl, timer):
        """Create the Overlap operator."""
        self.ksl = ksl
        self.timer = timer
        
    def orthonormalize(self, wfs, kpt, psit_nG=None):
        """Orthonormalizes the vectors a_nG with respect to the overlap.

        First, a Cholesky factorization C is done for the overlap
        matrix S_nn = <a_nG | S | a_nG> = C*_nn C_nn Cholesky matrix C
        is inverted and orthonormal vectors a_nG' are obtained as::

          psit_nG' = inv(C_nn) psit_nG
                    __
           ~   _   \    -1   ~   _
          psi (r) = )  C    psi (r)
             n     /__  nm     m
                    m

        Parameters
        ----------

        psit_nG: ndarray, input/output
            On input the set of vectors to orthonormalize,
            on output the overlap-orthonormalized vectors.
        kpt: KPoint object:
            k-point object from kpoint.py.
        work_nG: ndarray
            Optional work array for overlap matrix times psit_nG.
        work_nn: ndarray
            Optional work array for overlap matrix.

        """
        self.timer.start('Orthonormalize')
        if psit_nG is None:
            psit_nG = kpt.psit_nG
        P_ani = kpt.P_ani
        self.timer.start('projections')
        wfs.pt.integrate(psit_nG, P_ani, kpt.q)
        self.timer.stop('projections')

        # Construct the overlap matrix:
        operator = wfs.matrixoperator

        def S(psit_G):
            return psit_G
        
        def dS(a, P_ni):
            return np.dot(P_ni, wfs.setups[a].dO_ii)

        self.timer.start('calc_s_matrix')
        S_nn = operator.calculate_matrix_elements(psit_nG, P_ani, S, dS)
        self.timer.stop('calc_s_matrix')

        orthonormalization_string = repr(self.ksl)
        self.timer.start(orthonormalization_string)
        #
        if extra_parameters.get('sic', False):
            #
            # symmetric Loewdin Orthonormalization
            tri2full(S_nn, UL='L', map=np.conj)
            nrm_n = np.empty(S_nn.shape[0])
            diagonalize(S_nn, nrm_n)
            nrm_nn = np.diag(1.0/np.sqrt(nrm_n))
            S_nn = np.dot(np.dot(S_nn.T.conj(), nrm_nn), S_nn)
        else:
            #
            self.ksl.inverse_cholesky(S_nn)
        # S_nn now contains the inverse of the Cholesky factorization.
        # Let's call it something different:
        C_nn = S_nn
        del S_nn
        self.timer.stop(orthonormalization_string)

        self.timer.start('rotate_psi')
        operator.matrix_multiply(C_nn, psit_nG, P_ani, out_nG=kpt.psit_nG)
        self.timer.stop('rotate_psi')
        self.timer.stop('Orthonormalize')

    def apply(self, a_xG, b_xG, wfs, kpt, calculate_P_ani=True):
        """Apply the overlap operator to a set of vectors.

        Parameters
        ==========
        a_nG: ndarray
            Set of vectors to which the overlap operator is applied.
        b_nG: ndarray, output
            Resulting S times a_nG vectors.
        kpt: KPoint object
            k-point object defined in kpoint.py.
        calculate_P_ani: bool
            When True, the integrals of projector times vectors
            P_ni = <p_i | a_nG> are calculated.
            When False, existing P_ani are used

        """
        self.timer.start('Apply overlap')
        b_xG[:] = a_xG
        shape = a_xG.shape[:-3]
        P_axi = wfs.pt.dict(shape)

        if calculate_P_ani:
            wfs.pt.integrate(a_xG, P_axi, kpt.q)
        else:
            for a, P_ni in kpt.P_ani.items():
                P_axi[a][:] = P_ni

        for a, P_xi in P_axi.items():
            P_axi[a] = np.dot(P_xi, wfs.setups[a].dO_ii)
            # gemm(1.0, wfs.setups[a].dO_ii, P_xi, 0.0, P_xi, 'n')
        wfs.pt.add(b_xG, P_axi, kpt.q) # b_xG += sum_ai pt^a_i P_axi
        self.timer.stop('Apply overlap')

    def apply_inverse(self, a_xG, b_xG, wfs, kpt, calculate_P_ani=True):
        """Apply approximative inverse overlap operator to wave functions."""

        b_xG[:] = a_xG
        shape = a_xG.shape[:-3]
        P_axi = wfs.pt.dict(shape)

        if calculate_P_ani:
            wfs.pt.integrate(a_xG, P_axi, kpt.q)
        else:
            for a,P_ni in kpt.P_ani.items():
                P_axi[a][:] = P_ni

        for a, P_xi in P_axi.items():
            P_axi[a] = np.dot(P_xi, wfs.setups[a].dC_ii)
        wfs.pt.add(b_xG, P_axi, kpt.q)




# Copyright (C) 2003  CAMP
# Please see the accompanying LICENSE file for further information.

from math import pi

import numpy as np
from numpy.fft import fftn, ifftn, fft2, ifft2

from ase.parallel import parprint
from gpaw.transformers import Transformer
from gpaw.fd_operators import Laplace, LaplaceA, LaplaceB
from gpaw import PoissonConvergenceError
from gpaw.utilities.blas import axpy
from gpaw.utilities.gauss import Gaussian
from gpaw.utilities.ewald import madelung
from gpaw.utilities.tools import construct_reciprocal
import _gpaw

from gpaw import poisson

class PoissonSolver(poisson.PoissonSolver):
    """Copy of GPAW Poissonsolver"""

class FFTPoissonSolver(poisson.FFTPoissonSolver):

    def __init__(self, eps=2e-10, dtype=float):
        """Set the ``dtype`` of the source term array."""

        self.dtype = dtype
        self.charged_periodic_correction = None
        self.eps = eps

    def set_q(self, q_c):
        """Set q-vector in case of Bloch-type charge distribution.

        Parameters
        ----------
        q_c: ndarray
            q-vector in scaled coordinates of the reciprocal lattice vectors.

        """

        if self.gd.comm.rank == 0:
            self.k2_Q, self.N3 = construct_reciprocal(self.gd, q_c=q_c)

    def solve_neutral(self, phi_g, rho_g, eps=None):
        """Solve Poissons equation for a neutral and periodic charge density.

        Parameters
        ----------
        phi_g: ndarray
            Potential (output array).
        rho_g: ndarray
            Charge distribution (in units of -e).

        """

        assert phi_g.dtype == self.dtype
        assert rho_g.dtype == self.dtype

        if self.gd.comm.size == 1:
            # Note, implicit downcast from complex to float when the dtype of
            # phi_g is float
            phi_g[:] = ifftn(fftn(rho_g) * 4.0 * pi / self.k2_Q)
        else:
            rho_g = self.gd.collect(rho_g)
            if self.gd.comm.rank == 0:
                globalphi_g = ifftn(fftn(rho_g) * 4.0 * pi / self.k2_Q)
            else:
                globalphi_g = None
            # What happens here if globalphi is complex and phi is real ??????
            self.gd.distribute(globalphi_g, phi_g)

        return 1



from __future__ import print_function
from gpaw import GPAW, restart
from ase.structure import molecule
from gpaw.test import equal
Eini0 = -17.7436014436
Iini0 = 12
esolvers = ['cg', 'rmm-diis', 'dav']
E0 = {'cg':       -17.7439210822,
      'rmm-diis': -17.7439094546,
      'dav':      -17.7439760796}
I0 = {'cg': 6, 'rmm-diis': 6, 'dav': 8}


calc = GPAW(xc='LDA',
            eigensolver='cg',
            convergence={'eigenstates': 3.5e-5},
            #txt=None,
            dtype=complex)



mol = molecule('N2')
mol.center(vacuum=3.0)
mol.set_calculator(calc)

Eini = mol.get_potential_energy()
Iini = calc.get_number_of_iterations()
print(('%10s: %12.6f eV in %3d iterations' %
       ('init(cg)', Eini, Iini)))
equal(Eini, Eini0, 1E-8)

calc.write('N2_complex.gpw', mode='all')
del calc, mol

E = {}
I = {}
for esolver in esolvers:

    mol, calc = restart('N2_complex.gpw', txt=None)

    if (calc.wfs.dtype!=complex or
        calc.wfs.kpt_u[0].psit_nG.dtype!=complex):
        raise AssertionError('ERROR: restart failed to read complex WFS')
    
    calc.scf.reset()
    calc.set(convergence={'eigenstates': 3.5e-9})
    calc.set(eigensolver=esolver)

    E[esolver]=mol.get_potential_energy()
    I[esolver]=calc.get_number_of_iterations()
    
    print(('%10s: %12.6f eV in %3d iterations' %
           (esolver, E[esolver], I[esolver])))

for esolver in esolvers:
    print(esolver)
    equal(E[esolver], E0[esolver], 8E-5)
    



# -*- coding: utf-8 -*-
# Copyright (C) 2003-2015  CAMP
# Please see the accompanying LICENSE file for further information.

"""This module defines a Hamiltonian."""

import numpy as np

from gpaw.poisson import PoissonSolver
from gpaw.transformers import Transformer
from gpaw.lfc import LFC
from gpaw.utilities import pack2, unpack, unpack2
from gpaw.io import read_atomic_matrices
from gpaw.utilities.partition import AtomPartition
from gpaw.arraydict import ArrayDict


class Hamiltonian(object):
    """Hamiltonian object.

    Attributes:
     =============== =====================================================
     ``xc``          ``XC3DGrid`` object.
     ``poisson``     ``PoissonSolver``.
     ``gd``          Grid descriptor for coarse grids.
     ``finegd``      Grid descriptor for fine grids.
     ``restrict``    Function for restricting the effective potential.
     =============== =====================================================

    Soft and smooth pseudo functions on uniform 3D grids:
     ========== =========================================
     ``vHt_g``  Hartree potential on the fine grid.
     ``vt_sG``  Effective potential on the coarse grid.
     ``vt_sg``  Effective potential on the fine grid.
     ========== =========================================

    Energy contributions and forces:

    =========== ==========================================
                Description
    =========== ==========================================
    ``Ekin``    Kinetic energy.
    ``Epot``    Potential energy.
    ``Etot``    Total energy.
    ``Exc``     Exchange-Correlation energy.
    ``Eext``    Energy of external potential
    ``Eref``    Reference energy for all-electron atoms.
    ``S``       Entropy.
    ``Ebar``    Should be close to zero!
    =========== ==========================================

    """

    def __init__(self, gd, finegd, nspins, setups, timer, xc, world,
                 kptband_comm, grid2grid, vext=None, collinear=True):
        """Create the Hamiltonian."""
        self.gd = gd
        self.finegd = finegd
        self.nspins = nspins
        self.setups = setups
        self.timer = timer
        self.xc = xc
        self.collinear = collinear
        self.ncomp = 2 - int(collinear)
        self.ns = self.nspins * self.ncomp**2
        self.world = world
        self.kptband_comm = kptband_comm
        self.grid2grid = grid2grid
        self.aux_gd = grid2grid.big_gd if grid2grid.enabled else gd
        self.atomic_matrix_distributor = None
        self.dH_asp = None

        # The external potential
        self.vext = vext

        self.vt_sG = None
        self.vHt_g = None
        self.vt_sg = None

        self.atom_partition = None

        self.Ekin0 = None
        self.Ekin = None
        self.Epot = None
        self.Ebar = None
        self.Eext = None
        self.Exc = None
        self.Etot = None
        self.S = None

        self.ref_vt_sG = None
        self.ref_dH_asp = None

    @property
    def dH_asp(self):
        assert isinstance(self._dH_asp, ArrayDict) or self._dH_asp is None
        # self._dH_asp.check_consistency()
        return self._dH_asp

    @dH_asp.setter
    def dH_asp(self, value):
        if isinstance(value, dict):
            tmp = self.setups.empty_atomic_matrix(self.ns, self.atom_partition)
            tmp.update(value)
            value = tmp
        assert isinstance(value, ArrayDict) or value is None, type(value)
        if value is not None:
            value.check_consistency()
        self._dH_asp = value

    def summary(self, fd):
        fd.write('XC and Coulomb potentials evaluated on a %d*%d*%d grid\n' %
                 tuple(self.finegd.N_c))

    def set_positions_without_ruining_everything(self, spos_ac,
                                                 atom_partition):
        self.spos_ac = spos_ac
        rank_a = atom_partition.rank_a

        # If both old and new atomic ranks are present, start a blank dict if
        # it previously didn't exist but it will needed for the new atoms.
        # XXX what purpose does this serve?  In what case does it happen?
        # How would one even go about figuring it out?  Why does it all have
        # to be so unreadable? -Ask
        #
        if (self.atom_partition is not None and
            self.dH_asp is None and (rank_a == self.gd.comm.rank).any()):
            self.dH_asp = {}
            
        if self.atom_partition is not None and self.dH_asp is not None:
            self.timer.start('Redistribute')
            self.dH_asp.redistribute(atom_partition)
            self.timer.stop('Redistribute')

        self.atom_partition = atom_partition
        self.atomic_matrix_distributor = self.grid2grid.get_matrix_distributor(
            self.atom_partition, spos_ac)

    def set_positions(self, spos_ac, atom_partition):
        self.vbar.set_positions(spos_ac)
        self.xc.set_positions(spos_ac)
        self.set_positions_without_ruining_everything(spos_ac, atom_partition)

    def aoom(self, DM, a, l, scale=1):
        """Atomic Orbital Occupation Matrix.

        Determine the Atomic Orbital Occupation Matrix (aoom) for a
        given l-quantum number.

        This operation, takes the density matrix (DM), which for
        example is given by unpack2(D_asq[i][spin]), and corrects for
        the overlap between the selected orbitals (l) upon which the
        the density is expanded (ex <p|p*>,<p|p>,<p*|p*> ).

        Returned is only the "corrected" part of the density matrix,
        which represents the orbital occupation matrix for l=2 this is
        a 5x5 matrix.
        """
        S = self.setups[a]
        l_j = S.l_j
        lq = S.lq
        nl = np.where(np.equal(l_j, l))[0]
        V = np.zeros(np.shape(DM))
        if len(nl) == 2:
            aa = nl[0] * len(l_j) - (nl[0] - 1) * nl[0] // 2
            bb = nl[1] * len(l_j) - (nl[1] - 1) * nl[1] // 2
            ab = aa + nl[1] - nl[0]

            if not scale:
                lq_a = lq[aa]
                lq_ab = lq[ab]
                lq_b = lq[bb]
            else:
                lq_a = 1
                lq_ab = lq[ab] / lq[aa]
                lq_b = lq[bb] / lq[aa]

            # and the correct entrances in the DM
            nn = (2 * np.array(l_j) + 1)[0:nl[0]].sum()
            mm = (2 * np.array(l_j) + 1)[0:nl[1]].sum()

            # finally correct and add the four submatrices of NC_DM
            A = DM[nn:nn + 2 * l + 1, nn:nn + 2 * l + 1] * lq_a
            B = DM[nn:nn + 2 * l + 1, mm:mm + 2 * l + 1] * lq_ab
            C = DM[mm:mm + 2 * l + 1, nn:nn + 2 * l + 1] * lq_ab
            D = DM[mm:mm + 2 * l + 1, mm:mm + 2 * l + 1] * lq_b

            V[nn:nn + 2 * l + 1, nn:nn + 2 * l + 1] = lq_a
            V[nn:nn + 2 * l + 1, mm:mm + 2 * l + 1] = lq_ab
            V[mm:mm + 2 * l + 1, nn:nn + 2 * l + 1] = lq_ab
            V[mm:mm + 2 * l + 1, mm:mm + 2 * l + 1] = lq_b

            return A + B + C + D, V
        else:
            nn = (2 * np.array(l_j) + 1)[0:nl[0]].sum()
            A = DM[nn:nn + 2 * l + 1, nn:nn + 2 * l + 1] * lq[-1]
            V[nn:nn + 2 * l + 1, nn:nn + 2 * l + 1] = lq[-1]
            return A, V

    def initialize(self):
        self.vt_sg = self.finegd.empty(self.ns)
        self.vHt_g = self.finegd.zeros()
        self.vt_sG = self.gd.empty(self.ns)
        self.poisson.initialize()

    def update(self, density):
        """Calculate effective potential.

        The XC-potential and the Hartree potential are evaluated on
        the fine grid, and the sum is then restricted to the coarse
        grid."""

        self.timer.start('Hamiltonian')

        if self.vt_sg is None:
            self.timer.start('Initialize Hamiltonian')
            self.initialize()
            self.timer.stop('Initialize Hamiltonian')

        Epot, Ebar, Eext, Exc = self.update_pseudo_potential(density)
        Ekin_from_coarsegrid = self.calculate_kinetic_energy(density)
        W_aL = self.calculate_atomic_hamiltonians(density)
        Ekin, Epot, Ebar, Eext, Exc = self.update_corrections(
            density, 0.0, Epot, Ebar, Eext, Exc, W_aL)

        self.timer.start('Communicate')
        Ekin_from_coarsegrid = self.gd.comm.sum(Ekin_from_coarsegrid)
        if self.finegd.comm.rank == 0:
            Ekin += Ekin_from_coarsegrid
        energies = np.array([Ekin, Epot, Ebar, Eext, Exc])
        self.finegd.comm.sum(energies, 0)
        # Make sure that all CPUs have the same energies
        self.world.broadcast(energies, 0)
        self.timer.stop('Communicate')
        self.Ekin0, self.Epot, self.Ebar, self.Eext, self.Exc = energies
        self.timer.stop('Hamiltonian')

    def update_corrections(self, density, Ekin, Epot, Ebar, Eext, Exc, W_aL):
        self.timer.start('Atomic')
        self.dH_asp = None  # XXXX

        D_asp = self.atomic_matrix_distributor.distribute(density.D_asp)

        dH_asp = self.setups.empty_atomic_matrix(self.ns, D_asp.partition)
        for a, D_sp in D_asp.items():
            W_L = W_aL[a]
            setup = self.setups[a]

            D_p = D_sp[:self.nspins].sum(0)
            dH_p = (setup.K_p + setup.M_p +
                    setup.MB_p + 2.0 * np.dot(setup.M_pp, D_p) +
                    np.dot(setup.Delta_pL, W_L))
            Ekin += np.dot(setup.K_p, D_p) + setup.Kc
            Ebar += setup.MB + np.dot(setup.MB_p, D_p)
            Epot += setup.M + np.dot(D_p, (setup.M_p +
                                           np.dot(setup.M_pp, D_p)))

            dH_asp[a] = dH_sp = np.zeros_like(D_sp)

            if setup.HubU is not None:
                assert self.collinear
                nspins = len(D_sp)

                l_j = setup.l_j
                l = setup.Hubl
                scale = setup.Hubs
                nl = np.where(np.equal(l_j, l))[0]
                nn = (2 * np.array(l_j) + 1)[0:nl[0]].sum()

                for D_p, H_p in zip(D_sp, dH_asp[a]):
                    [N_mm, V] = self.aoom(unpack2(D_p), a, l, scale)
                    N_mm = N_mm / 2 * nspins

                    Eorb = setup.HubU / 2. * (N_mm -
                                              np.dot(N_mm, N_mm)).trace()
                    Vorb = setup.HubU * (0.5 * np.eye(2 * l + 1) - N_mm)
                    Exc += Eorb
                    if nspins == 1:
                        # add contribution of other spin manyfold
                        Exc += Eorb

                    if len(nl) == 2:
                        mm = (2 * np.array(l_j) + 1)[0:nl[1]].sum()

                        V[nn:nn + 2 * l + 1, nn:nn + 2 * l + 1] *= Vorb
                        V[mm:mm + 2 * l + 1, nn:nn + 2 * l + 1] *= Vorb
                        V[nn:nn + 2 * l + 1, mm:mm + 2 * l + 1] *= Vorb
                        V[mm:mm + 2 * l + 1, mm:mm + 2 * l + 1] *= Vorb
                    else:
                        V[nn:nn + 2 * l + 1, nn:nn + 2 * l + 1] *= Vorb

                    Htemp = unpack(H_p)
                    Htemp += V
                    H_p[:] = pack2(Htemp)

            dH_sp[:self.nspins] += dH_p
            if self.ref_dH_asp:
                dH_sp += self.ref_dH_asp[a]
            # We are not yet done with dH_sp; still need XC correction below

        # It should now be possible to do the XC correction in the
        # same loop as above........
        self.timer.start('XC Correction')
        for a, D_sp in D_asp.items():
            setup = self.setups[a]
            dH_sp = dH_asp[a]
            Exc += self.xc.calculate_paw_correction(setup, D_sp, dH_sp, a=a)
        self.timer.stop('XC Correction')

        for a, D_sp in D_asp.items():
            Ekin -= (D_sp * dH_asp[a]).sum()  # NCXXX
        self.dH_asp = self.atomic_matrix_distributor.collect(dH_asp)
        self.timer.stop('Atomic')

        # Make corrections due to non-local xc:
        self.Enlxc = 0.0  # XXXxcfunc.get_non_local_energy()
        Ekin += self.xc.get_kinetic_energy_correction() / self.finegd.comm.size
        return Ekin, Epot, Ebar, Eext, Exc

    def get_energy(self, occupations):
        self.Ekin = self.Ekin0 + occupations.e_band
        self.S = occupations.e_entropy

        # Total free energy:
        self.Etot = (self.Ekin + self.Epot + self.Eext +
                     self.Ebar + self.Exc - self.S)

        return self.Etot

    def linearize_to_xc(self, new_xc, density):
        # Store old hamiltonian
        ref_vt_sG = self.vt_sG.copy()
        ref_dH_asp = {}
        for a, dH_sp in self.dH_asp.items():
            ref_dH_asp[a] = dH_sp.copy()
        self.xc = new_xc
        self.xc.set_positions(self.spos_ac)
        self.update(density)

        ref_vt_sG -= self.vt_sG
        for a, dH_sp in self.dH_asp.items():
            ref_dH_asp[a] -= dH_sp
        self.ref_vt_sG = ref_vt_sG
        self.ref_dH_asp = ref_dH_asp

    def calculate_forces(self, dens, F_av):
        ghat_aLv = dens.ghat.dict(derivative=True)
        nct_av = dens.nct.dict(derivative=True)
        vbar_av = self.vbar.dict(derivative=True)

        self.calculate_forces2(dens, ghat_aLv, nct_av, vbar_av)
        F_coarsegrid_av = np.zeros_like(F_av)

        # Force from compensation charges:
        for a, dF_Lv in ghat_aLv.items():
            F_av[a] += np.dot(dens.Q_aL[a], dF_Lv)

        # Force from smooth core charge:
        for a, dF_v in nct_av.items():
            F_coarsegrid_av[a] += dF_v[0]

        # Force from zero potential:
        for a, dF_v in vbar_av.items():
            F_av[a] += dF_v[0]

        self.xc.add_forces(F_av)
        self.gd.comm.sum(F_coarsegrid_av, 0)
        self.finegd.comm.sum(F_av, 0)
        F_av += F_coarsegrid_av

    def apply_local_potential(self, psit_nG, Htpsit_nG, s):
        """Apply the Hamiltonian operator to a set of vectors.

        XXX Parameter description is deprecated!

        Parameters:

        a_nG: ndarray
            Set of vectors to which the overlap operator is applied.
        b_nG: ndarray, output
            Resulting H times a_nG vectors.
        kpt: KPoint object
            k-point object defined in kpoint.py.
        calculate_projections: bool
            When True, the integrals of projector times vectors
            P_ni = <p_i | a_nG> are calculated.
            When False, existing P_uni are used
        local_part_only: bool
            When True, the non-local atomic parts of the Hamiltonian
            are not applied and calculate_projections is ignored.

        """
        vt_G = self.vt_sG[s]
        if psit_nG.ndim == 3:
            Htpsit_nG += psit_nG * vt_G
        else:
            for psit_G, Htpsit_G in zip(psit_nG, Htpsit_nG):
                Htpsit_G += psit_G * vt_G

    def apply(self, a_xG, b_xG, wfs, kpt, calculate_P_ani=True):
        """Apply the Hamiltonian operator to a set of vectors.

        Parameters:

        a_nG: ndarray
            Set of vectors to which the overlap operator is applied.
        b_nG: ndarray, output
            Resulting S times a_nG vectors.
        wfs: WaveFunctions
            Wave-function object defined in wavefunctions.py
        kpt: KPoint object
            k-point object defined in kpoint.py.
        calculate_P_ani: bool
            When True, the integrals of projector times vectors
            P_ni = <p_i | a_nG> are calculated.
            When False, existing P_ani are used

        """

        wfs.kin.apply(a_xG, b_xG, kpt.phase_cd)
        self.apply_local_potential(a_xG, b_xG, kpt.s)
        shape = a_xG.shape[:-3]
        P_axi = wfs.pt.dict(shape)

        if calculate_P_ani:  # TODO calculate_P_ani=False is experimental
            wfs.pt.integrate(a_xG, P_axi, kpt.q)
        else:
            for a, P_ni in kpt.P_ani.items():
                P_axi[a][:] = P_ni

        for a, P_xi in P_axi.items():
            dH_ii = unpack(self.dH_asp[a][kpt.s])
            P_axi[a] = np.dot(P_xi, dH_ii)
        wfs.pt.add(b_xG, P_axi, kpt.q)

    def get_xc_difference(self, xc, density):
        """Calculate non-selfconsistent XC-energy difference."""
        if density.nt_sg is None:
            density.interpolate_pseudo_density()
        nt_sg = density.nt_sg
        if hasattr(xc, 'hybrid'):
            xc.calculate_exx()
        Exc = xc.calculate(density.finegd, nt_sg) / self.finegd.comm.size
        D_asp = self.atomic_matrix_distributor.distribute(density.D_asp)
        for a, D_sp in D_asp.items():
            setup = self.setups[a]
            Exc += xc.calculate_paw_correction(setup, D_sp)
        Exc = self.finegd.comm.sum(Exc)
        return Exc - self.Exc

    def estimate_memory(self, mem):
        nbytes = self.gd.bytecount()
        nfinebytes = self.finegd.bytecount()
        arrays = mem.subnode('Arrays', 0)
        arrays.subnode('vHt_g', nfinebytes)
        arrays.subnode('vt_sG', self.nspins * nbytes)
        arrays.subnode('vt_sg', self.nspins * nfinebytes)
        self.xc.estimate_memory(mem.subnode('XC'))
        self.poisson.estimate_memory(mem.subnode('Poisson'))
        self.vbar.estimate_memory(mem.subnode('vbar'))

    def read(self, reader, parallel):
        self.Ekin = reader['Ekin']
        self.Epot = reader['Epot']
        self.Ebar = reader['Ebar']
        try:
            self.Eext = reader['Eext']
        except (AttributeError, KeyError):
            self.Eext = 0.0
        self.Exc = reader['Exc']
        self.S = reader['S']
        self.Etot = reader.get('PotentialEnergy',
                               broadcast=True) - 0.5 * self.S

        if not reader.has_array('PseudoPotential'):
            return

        hdf5 = hasattr(reader, 'hdf5')
        version = reader['version']

        # Read pseudo potential on the coarse grid
        # and broadcast on kpt/band comm:
        if version > 0.3:
            # XXX should this not be self.ns instead of self.nspins?
            self.vt_sG = self.gd.empty(self.nspins)
            if hdf5:
                indices = [slice(0, self.nspins), ] + self.gd.get_slice()
                do_read = (self.kptband_comm.rank == 0)
                reader.get('PseudoPotential', out=self.vt_sG,
                           parallel=parallel,
                           read=do_read, *indices)  # XXX read=?
                self.kptband_comm.broadcast(self.vt_sG, 0)
            else:
                for s in range(self.nspins):
                    self.gd.distribute(reader.get('PseudoPotential', s),
                                       self.vt_sG[s])

        self.atom_partition = AtomPartition(self.gd.comm,
                                            np.zeros(len(self.setups), int))

        # Read non-local part of hamiltonian
        self.dH_asp = {}
        if version > 0.3:
            all_H_sp = reader.get('NonLocalPartOfHamiltonian', broadcast=True)

        if self.gd.comm.rank == 0 and version > 0.3:
            self.dH_asp = read_atomic_matrices(all_H_sp, self.setups)


class RealSpaceHamiltonian(Hamiltonian):
    def __init__(self, gd, finegd, nspins, setups, timer, xc, world,
                 kptband_comm, vext=None, collinear=True,
                 psolver=None, stencil=3, grid2grid=None):
        Hamiltonian.__init__(self, gd, finegd, nspins, setups, timer, xc,
                             world, kptband_comm, vext=vext,
                             collinear=collinear, grid2grid=grid2grid)

        # Solver for the Poisson equation:
        if psolver is None:
            psolver = PoissonSolver(nn=3, relax='J')
        self.poisson = psolver
        self.poisson.set_grid_descriptor(self.finegd)

        # Restrictor function for the potential:
        self.restrictor = Transformer(self.finegd, self.aux_gd, stencil)
        self.restrict = self.restrictor.apply

        self.vbar = LFC(self.finegd, [[setup.vbar] for setup in setups],
                        forces=True)
        self.vbar_g = None

    def restrict_and_collect(self, a_xg, b_xg=None, phases=None):
        if self.grid2grid.enabled:
            tmp_xg = self.restrictor.apply(a_xg, output=None, phases=phases)
            b_xg = self.grid2grid.collect(tmp_xg, b_xg)
        else:
            b_xg = self.restrictor.apply(a_xg, output=b_xg, phases=phases)
        return b_xg

    def summary(self, fd):
        Hamiltonian.summary(self, fd)

        degree = self.restrictor.nn * 2 - 1
        name = ['linear', 'cubic', 'quintic', 'heptic'][degree // 2]
        fd.write('Interpolation: tri-%s ' % name +
                 '(%d. degree polynomial)\n' % degree)

        fd.write('Poisson solver: %s\n' % self.poisson.get_description())

    def set_positions(self, spos_ac, rank_a):
        Hamiltonian.set_positions(self, spos_ac, rank_a)
        if self.vbar_g is None:
            self.vbar_g = self.finegd.empty()
        self.vbar_g[:] = 0.0
        self.vbar.add(self.vbar_g)

    def update_pseudo_potential(self, density):
        self.timer.start('vbar')
        Ebar = self.finegd.integrate(self.vbar_g, density.nt_g,
                                     global_integral=False)

        vt_g = self.vt_sg[0]
        vt_g[:] = self.vbar_g
        self.timer.stop('vbar')

        Eext = 0.0
        if self.vext is not None:
            assert self.collinear
            vext_g = self.vext.get_potential(self.finegd)
            vt_g += vext_g
            Eext = self.finegd.integrate(vext_g, density.rhot_g,
                                         global_integral=False)

        self.vt_sg[1:self.nspins] = vt_g

        self.vt_sg[self.nspins:] = 0.0

        self.timer.start('XC 3D grid')
        Exc = self.xc.calculate(self.finegd, density.nt_sg, self.vt_sg)
        Exc /= self.finegd.comm.size
        self.timer.stop('XC 3D grid')

        self.timer.start('Poisson')
        # npoisson is the number of iterations:
        self.npoisson = self.poisson.solve(self.vHt_g, density.rhot_g,
                                           charge=-density.charge)
        self.timer.stop('Poisson')

        self.timer.start('Hartree integrate/restrict')
        Epot = 0.5 * self.finegd.integrate(self.vHt_g, density.rhot_g,
                                           global_integral=False)

        for vt_g in self.vt_sg[:self.nspins]:
            vt_g += self.vHt_g

        self.timer.stop('Hartree integrate/restrict')
        return Epot, Ebar, Eext, Exc

    def calculate_kinetic_energy(self, density):
        # XXX new timer item for kinetic energy?
        self.timer.start('Hartree integrate/restrict')
        self.restrict_and_collect(self.vt_sg, self.vt_sG)
        
        Ekin = 0.0
        s = 0
        for vt_G, nt_G in zip(self.vt_sG, density.nt_sG):
            if self.ref_vt_sG is not None:
                vt_G += self.ref_vt_sG[s]

            if s < self.nspins:
                Ekin -= self.gd.integrate(vt_G, nt_G - density.nct_G,
                                          global_integral=False)
            else:
                Ekin -= self.gd.integrate(vt_G, nt_G, global_integral=False)
            s += 1
        self.timer.stop('Hartree integrate/restrict')
        return Ekin

    def calculate_atomic_hamiltonians(self, dens):
        W_aL = dens.ghat.dict()
        if self.vext:
            vext_g = self.vext.get_potential(self.finegd)
            dens.ghat.integrate(self.vHt_g + vext_g, W_aL)
        else:
            dens.ghat.integrate(self.vHt_g, W_aL)
        return W_aL

    def calculate_forces2(self, dens, ghat_aLv, nct_av, vbar_av):
        if self.nspins == 2:
            vt_G = self.vt_sG.mean(0)
        else:
            vt_G = self.vt_sG[0]

        self.vbar.derivative(dens.nt_g, vbar_av)
        if self.vext:
            vext_g = self.vext.get_potential(self.finegd)
            dens.ghat.derivative(self.vHt_g + vext_g, ghat_aLv)
        else:
            dens.ghat.derivative(self.vHt_g, ghat_aLv)
        dens.nct.derivative(vt_G, nct_av)



# Test that the atomic corrections of LCAO work correctly,
# by verifying that the different implementations yield the same numbers.
#
# For example the corrections P^* dH P to the Hamiltonian.
#
# This is done by invoking GPAW once for each type of calculation.

from ase.structure import molecule
from gpaw import GPAW, LCAO, PoissonSolver
from gpaw.lcao.atomic_correction import DenseAtomicCorrection, \
    DistributedAtomicCorrection, ScipyAtomicCorrection
from gpaw.mpi import world

# Use a cell large enough that some overlaps are zero.
# Thus the matrices will have at least some sparsity.
system = molecule('H2O')
system.center(vacuum=3.0)
system.pbc = (0, 0, 1)
system = system.repeat((1, 1, 2))
# Break symmetries so we don't get funny degeneracy effects.
system.rattle(stdev=0.05)

corrections = [DenseAtomicCorrection(), DistributedAtomicCorrection()]

#corrections.pop() # XXXXXXXXXXXXXXXXXXXXXXXXXXXX
try:
    import scipy
except ImportError:
    pass
else:
    corrections.append(ScipyAtomicCorrection(tolerance=0.0))

energies = []
for correction in corrections:
    parallel = {}
    if world.size >= 4:
        parallel['band'] = 2
    if correction.name != 'dense':
        parallel['sl_auto'] = True
    calc = GPAW(mode=LCAO(atomic_correction=correction),
                basis='sz(dzp)',
                #kpts=(1, 1, 4),
                #spinpol=True,
                poissonsolver=PoissonSolver(relax='J', eps=1e100, nn=1),
                parallel=parallel,
                h=0.35)
    def stopcalc():
        calc.scf.converged = True
    calc.attach(stopcalc, 2)
    system.set_calculator(calc)
    energy = system.get_potential_energy()
    energies.append(energy)

master = calc.wfs.world.rank == 0
if master:
    print('energies', energies)

eref = energies[0]
errs = []
for energy, c in zip(energies, corrections):
    err = abs(energy - eref)
    nops = calc.wfs.world.sum(c.nops)
    errs.append(err)
    if master:
        print('err=%e :: name=%s :: nops=%d' % (err, c.name, nops))

assert max(errs) < 1e-11



"""Helper classes for doing jellium calculations."""

from math import pi

import numpy as np
from ase.units import Bohr

from gpaw.poisson import PoissonSolver


class JelliumPoissonSolver(PoissonSolver):
    """Jellium Poisson solver."""
    
    mask_g = None  # where to put the jellium
    rs = None  # Wigner Seitz radius
    
    def get_mask(self, r_gv):
        """Choose which grid points are inside the jellium.

        r_gv: 4-dimensional ndarray
            positions of the grid points in Bohr units.

        Return ndarray of ones and zeros indicating where the jellium
        is.  This implementation will put the positive background in the
        whole cell.  Overwrite this method in subclasses."""
        
        return self.gd.zeros() + 1.0
        
    def initialize(self):
        PoissonSolver.initialize(self)
        r_gv = self.gd.get_grid_point_coordinates().transpose((1, 2, 3, 0))
        self.mask_g = self.get_mask(r_gv).astype(float)
        self.volume = self.gd.comm.sum(self.mask_g.sum()) * self.gd.dv
        
    def solve(self, phi, rho, eps=None, charge=0, maxcharge=1e-6,
              zero_initial_phi=False):

        if eps is None:
            eps = self.eps
        
        self.rs = (3 / pi / 4 * self.volume / charge)**(1 / 3.0)
        
        rho -= self.mask_g * (charge / self.volume)
        niter = self.solve_neutral(phi, rho, eps=eps)
        return niter


class JelliumSurfacePoissonSolver(JelliumPoissonSolver):
    def __init__(self, z1, z2, **kwargs):
        """Put the positive background charge where z1 < z < z2.

        z1: float
            Position of lower surface in Angstrom units.
        z2: float
            Position of upper surface in Angstrom units."""
        
        PoissonSolver.__init__(self, **kwargs)
        self.z1 = (z1 - 0.0001) / Bohr
        self.z2 = (z2 - 0.0001) / Bohr

    def get_mask(self, r_gv):
        return np.logical_and(r_gv[:, :, :, 2] > self.z1,
                              r_gv[:, :, :, 2] < self.z2)



import numpy as np
from gpaw.mpi import size, rank, world
from gpaw.response.parallel import par_write, par_read

Nw = 400
npw = 10
assert Nw % size == 0
Nw_local = Nw // size

chi0_wGG = np.ones((Nw_local, npw, npw),dtype=complex) * rank

filename = 'chi0'
name = 'chi0_wGG'
par_write(filename, name, world, chi0_wGG)

chi0_wGG_new = par_read(filename, name)

assert (np.abs(chi0_wGG - chi0_wGG_new) < 1e-10).all()



"""Compare TPSS from scratch and from PBE"""
from ase import Atoms
from gpaw import GPAW

n = Atoms('N', magmoms=[3])
n.center(vacuum=2.5)
n.calc = GPAW(xc='TPSS')
e1 = n.get_potential_energy()

n.calc = GPAW(xc='PBE')
n.get_potential_energy()
n.calc.set(xc='TPSS')
e2 = n.get_potential_energy()
print('Energy difference', e1 - e2)
assert abs(e1 - e2) < 1e-5



"""Test automagical calculation of wfs"""

import os
import sys
import time
from gpaw import GPAW
from ase import Atom, Atoms
from gpaw.test import equal
from ase.parallel import rank, barrier, size

txt=None
ending = 'gpw'
restart = 'gpaw-restart-wfs_auto.' + ending

# H2
H = Atoms([Atom('H', (0, 0, 0)), Atom('H', (0, 0, 1))])
H.center(vacuum=2.0)

calc = GPAW(nbands=2, convergence={'eigenstates': 1e-3},
            txt=txt)
H.set_calculator(calc)
H.get_potential_energy()
calc.write(restart)

calc = GPAW(restart)
calc.set(fixdensity=False) # provoke deletion of calc.scf
calc.converge_wave_functions()

calc.set(nbands=5)
calc.converge_wave_functions()

if rank == 0:
    os.remove(restart)



from __future__ import print_function
import os
import pickle
import numpy as np
import matplotlib.pyplot as pl
import smtplib

try:
    from email.mime.text import MIMEText
    from email.mime.multipart import MIMEMultipart
except ImportError:
    from email.MIMEMultipart import MIMEMultipart
    from email.MIMEText import MIMEText

"""Database structure:
dict(testname: [(rev, runtime, info), (rev, runtime, info), ...])
    rev: SVN revision
    runtime: Run time in seconds. Negative for crashed jobs!
    info: A string describing the outcome
"""


class DatabaseHandler:
    """Database class for keeping timings and info for long tests"""
    def __init__(self, filename):
        self.filename = filename
        self.data = dict()

    def read(self):
        if os.path.isfile(self.filename):
            self.data = pickle.load(file(self.filename))
        else:
            print('File does not exist, starting from scratch')

    def write(self, filename=None):
        if filename is None:
            filename = self.filename
        if os.path.isfile(filename):
            os.rename(filename, filename + '.old')
        pickle.dump(self.data, open(filename, 'wb'))

    def add_data(self, name, rev, runtime, info):
        if name not in self.data:
            self.data[name] = []
        self.data[name].append((rev, runtime, info))

    def get_data(self, name):
        """Return rev, time_array"""
        revs, runtimes = [], []
        if name in self.data:
            for datapoint in self.data[name]:
                revs.append(datapoint[0])
                runtimes.append(datapoint[1])

        return np.asarray(revs), np.asarray(runtimes)

    def update(self, queue, rev):
        """Add all new data to database"""
        for job in queue.jobs:
            absname = job.absname

            tstart = job.tstart
            if tstart is None:
                tstart = np.nan
            tstop = job.tstop
            if tstop is None:
                tstop = np.nan

            info = job.status

            self.add_data(absname, rev, tstop - tstart, info)


class TestAnalyzer:
    def __init__(self, name, revs, runtimes):
        self.name = name
        self.revs = revs
        self.runtimes = runtimes
        self.better = []
        self.worse = []
        self.relchange = None
        self.abschange = None

    def analyze(self, reltol=0.1, abstol=5.0):
        """Analyze timings

        When looking at a point, attention is needed if it deviates more than
        10\% from the median of previous points. If such a point occurs the
        analysis is restarted.
        """
        self.better = []
        self.worse = []
        abschange = 0.0
        relchange = 0.0
        status = 0
        current_first = 0   # Point to start analysis from
        for i in range(1, len(self.runtimes)):
            tmpruntimes = self.runtimes[current_first:i]
            median = np.median(tmpruntimes[np.isfinite(tmpruntimes)])
            if np.isnan(median):
                current_first = i
            elif np.isfinite(self.runtimes[i]):
                abschange = self.runtimes[i] - median
                relchange = abschange / median
                if relchange < -reltol and abschange < -abstol:
                    # Improvement
                    current_first = i
                    self.better.append(i)
                    status = -1
                elif relchange > reltol and abschange > abstol:
                    # Regression
                    current_first = i
                    self.worse.append(i)
                    status = 1
                else:
                    status = 0

        self.status = status
        self.abschange = abschange
        self.relchange = relchange * 100

    def plot(self, outputdir=None):
        if outputdir is None:
            return
        fig = pl.figure()
        ax = fig.add_subplot(1, 1, 1)
        ax.plot(self.revs, self.runtimes, 'ko-')
        ax.plot(self.revs[self.better],
                self.runtimes[self.better],
                'go', markersize=8)
        ax.plot(self.revs[self.worse],
                self.runtimes[self.worse],
                'ro', markersize=8)
        ax.set_title(self.name)
        if not outputdir.endswith('/'):
            outputdir += '/'
        figname = self.name.replace('/', '_')
        fig.savefig(outputdir + figname + '.png')

        
class MailGenerator:
    def __init__(self, queue):
        self.better = []
        self.worse = []

        self.FAILED = []
        self.TIMEOUT = []

    def add_test(self, ta):
        if ta.abschange < 0.0:
            self.add_better(ta)
        else:
            self.add_worse(ta)

    def add_better(self, ta):
        self.better.append((ta.name, ta.runtimes[-1],
                            ta.abschange, ta.relchange))

    def add_worse(self, ta):
        self.worse.append((ta.name, ta.runtimes[-1],
                           ta.abschange, ta.relchange))

    def add_failed(self, name):
        self.FAILED.append(name)

    def add_timeout(self, name):
        self.TIMEOUT.append(name)

    def generate_mail(self):
        mail = ''
        if len(self.FAILED):
            mail += 'The following %i tests failed:\n' % len(self.FAILED)
            for name in self.FAILED:
                mail += '%s\n' % name
            mail += '\n'

        if len(self.TIMEOUT):
            mail += 'The following %i tests timed out:\n' % len(self.TIMEOUT)
            for name in self.TIMEOUT:
                mail += '%s\n' % name
            mail += '\n'

        if len(self.FAILED) or len(self.TIMEOUT):
            mail += 'See attached log for details\n\n'

        mail += 'Benchmark results from weekly tests:\n\n'

        if len(self.better):
            mail += 'The following %i tests improved:\n' % len(self.better)
            for test in self.better:
                mail += '%-55s %7.1fs:%+8.1fs (%+7.2f%%)\n' % test
        else:
            mail += 'No tests improved!\n'

        mail += '\n'

        if len(self.worse):
            mail += 'The following %i tests regressed:\n' % len(self.worse)
            for test in self.worse:
                mail += '%-55s %7.1fs:%+8.1fs (%+7.2f%%)\n' % test
        else:
            mail += 'No tests regressed!\n'

        return mail

    def generate_subject(self):
        subject = 'Weekly tests: '
        if len(self.FAILED):
            subject += '%i FAILURES, ' % len(self.FAILED)
        if len(self.TIMEOUT):
            subject += '%i TIMEOUTS, ' % len(self.TIMEOUT)
        subject += '%i regressions, ' % len(self.worse)
        subject += '%i improvements.' % len(self.better)
        return subject

    def send_mail(self, address, server, attachment=None):
        msg = MIMEMultipart()
        msg['Subject'] = self.generate_subject()
        me = os.environ['USER'] + '@' + server
        msg['From'] = me
        msg['To'] = address
        msg.attach(MIMEText(self.generate_mail()))

        if attachment:
            a = MIMEText(open(attachment).read())
            a.add_header('Content-Disposition', 'attachment',
                         filename='status.log')
            msg.attach(a)
        
        s = smtplib.SMTP(server)
        s.sendmail(me, [address], msg.as_string())
        s.quit()


#def csv2database(infile, outfile):
#    """Use this file once to import the old data from csv"""
#    csvdata = np.recfromcsv(infile)
#    db = DatabaseHandler(outfile)
#    for test in csvdata:
#        name = test[0]
#        for i in range(1, len(test) - 1):
#            runtime = float(test[i])
#            info = ''
#            db.add_data(name, 0, runtime, info)
#    db.write()

def analyse(queue, dbpath, outputdir=None, rev=None,
            mailto=None, mailserver=None, attachment=None):
    """Analyse runtimes from testsuite

    Parameters:
        queue: AGTSQueue
            Que to analuze
        dbpath: str
            Path to file storing previous results
        outputdir: str|None
            If str, figures will be put in this dir
        rev: int|None
            GPAW revision. If None time.time() is used
        mailto: str|None
            Mailaddres to send results to. If None, results will be printed to
            stdout.
    """
    if rev is None:
        import time
        rev = time.time()
    db = DatabaseHandler(dbpath)
    db.read()
    db.update(queue, rev)
    db.write()
    mg = MailGenerator(queue)
    for job in queue.jobs:
        name = job.absname
        if job.status == 'success':
            revs, runtimes = db.get_data(name)
            ta = TestAnalyzer(name, revs, runtimes)
            ta.analyze(abstol=25, reltol=0.04)
            if ta.status:
                mg.add_test(ta)
            ta.plot(outputdir)
        elif job.status == 'FAILED':
            mg.add_failed(name)
        elif job.status == 'TIMEOUT':
            mg.add_timeout(name)

    if mailto is not None:
        mg.send_mail(mailto, mailserver, attachment)
    else:
        print(mg.generate_subject())
        print(mg.generate_mail())



# Note: due to how agts works external executables (abinit, aims, etc.)
# must be run on the submission platform (bmaster1), currently opteron4
def agts(queue):
    queue.add('pbe_abinit_paw.py Au', ncpus=1,
              queueopts='-l nodes=1:ppn=4:opteron4', walltime=10*60)



from __future__ import print_function
import numpy as np
from time import time, ctime
from datetime import timedelta
from ase.lattice import bulk
from ase.units import Hartree
from gpaw import GPAW, FermiDirac
from gpaw.response.gw import GW

starttime = time()

a = 3.567
atoms = bulk('C', 'diamond', a=a)

kpts = (2,2,2)

calc = GPAW(
            h=0.20,
            kpts=kpts,
            xc='LDA',
            txt='C_gs.txt',
            nbands=30,
            symmetry='off',
            convergence={'bands':25},
            eigensolver='cg',
            occupations=FermiDirac(0.001)
           )

atoms.set_calculator(calc)
atoms.get_potential_energy()
calc.write('C_gs.gpw','all')

file='C_gs.gpw'

gw = GW(
        file=file,
        nbands=25,
        bands=np.array([3,4]),
        w=np.array([75., 100., 0.05]),
        ecut=25.,
        eta=0.2,
        ppa=False,
        hilbert_trans=False,
       )

gw.get_exact_exchange()
gw.get_QP_spectrum()

QP_False = gw.QP_skn * Hartree

gw = GW(
        file=file,
        nbands=25,
        bands=np.array([3,4]),
        w=np.array([75., 100., 0.05]),
        ecut=25.,
        eta=0.2,
        ppa=True,
        hilbert_trans=False,
       )

gw.get_QP_spectrum()

QP_True = gw.QP_skn * Hartree

if not (np.abs(QP_False - QP_True) < 0.05).all():
    raise AssertionError("dynamic GW not equal to PPA")

totaltime = round(time() - starttime)
print("GW test finished in %s " %(timedelta(seconds=totaltime)))



from __future__ import print_function
from ase import *
from ase.dft.kpoints import monkhorst_pack
from ase.lattice import bulk
from gpaw import *
from gpaw.test import equal
import numpy as np
from gpaw.mpi import serial_comm, size, rank, world

atoms = bulk('Al', 'fcc')
kpts = monkhorst_pack((4,4,4))
kpts += np.array([1/8., 1/8., 1/8.])

calc = GPAW(h=0.18,
            kpts=kpts,
            occupations=FermiDirac(0.1))
atoms.set_calculator(calc)
E = atoms.get_potential_energy()
calc.write('Al.gpw','all') 

calc = GPAW('Al.gpw',txt=None)
E = calc.get_potential_energy()

from gpaw.xc.hybridk import HybridXC
exx = HybridXC('EXX',acdf=True)
E_k = E + calc.get_xc_difference(exx)

if 0:#size == 1:
    calc = GPAW('Al.gpw',txt=None, communicator=serial_comm)
    from gpaw.xc.hybridq import HybridXC
    exx = HybridXC('EXX')
    E_q = E + calc.get_xc_difference(exx)

    print(E_q, E_k)
    equal(E_q, E_k, 0.001)
equal(E_k, -15.6, 0.1)



"""Different kinds of laser pulses.
"""

import sys
import time
from math import log

import numpy as np
from ase.units import Bohr, Hartree

import gpaw.io
from gpaw.tddft.units import attosec_to_autime, autime_to_attosec, \
                             eV_to_aufrequency, aufrequency_to_eV

###########################
# Main class
###########################
class LaserField:
  """
    
  """    
  def __init__(self):
      pass
  
  def strength(self, t):
      return np.array([0.0, 0.0, 0.0])



class CWField(LaserField):
  """
  Continuously oscillating laser field which is switch on linearly.

  Parameters:
    e0   field strength  (in atomic units)
    w    field frequency (in atomic units)
    ts   switch on time  (in atomic units)    
  """
  def __init__(self, e0, w, ts):
    # FIXME: use eV, ang and attosec
    #           attosec_to_autime, autime_to_attosec, \
    #           eV_to_aufrequency, aufrequency_to_eV
    self.e0 = e0
    self.w  = w
    self.ts = ts
    
  def strength(self, t):
    if t < self.ts:
      c = self.e0 * (t / self.ts) * np.sin(self.w * t)
    else:
      c = self.e0 * np.sin(self.w * t)
    return np.array([0.0, 0.0, c])



from __future__ import print_function
import os
import fnmatch
import urllib2
from StringIO import StringIO
import tarfile
from optparse import OptionParser, OptionGroup
import re


usage = '%prog [OPTION]\n  or:  %prog [OPTION] INSTALLDIR'
description = ('In first form, show available setups and GPAW setup paths.  '
               'In second form, download and install gpaw-setups into '
               'INSTALLDIR/[setups-package-name-and-version].')


sources = [('gpaw', 'official GPAW setups releases [default]'),
           ('sg15', 'SG15 pseudopotentials'),
           ('basis', 'basis sets for LCAO mode'),
           ('test', 'small file for testing this script'),
           ]
names = [r for r, d in sources]


def main(args):
    p = OptionParser(usage=usage, description=description)
    add = p.add_option
    add('--version', metavar='VERSION',
        help='download VERSION of package.  '
        'Run without arguments to display a list of versions.  '
        'VERSION can be the full URL or a part such as  '
        '\'0.8\' or \'0.6.6300\'')
    add('--tarball', metavar='FILE',
        help='unpack and install from local tarball FILE '
        'instead of downloading')
    add('--list-all', action='store_true',
        help='list packages from all sources')
    g = OptionGroup(p, 'Sources')
    for name, help in sources:
        g.add_option('--%s' % name, action='store_const',
                     const=name, dest='source',
                     help=help)
    p.add_option_group(g)
    add('--register', action='store_true',
        help='run non-interactively and register install path in '
        'GPAW setup search paths.  This is done by adding lines to '
        '~/.gpaw/rc.py')
    add('--no-register', action='store_true',
        help='run non-interactively and do not register install path in '
        'GPAW setup search paths')
    opts, args = p.parse_args(args)
    nargs = len(args)
    
    if opts.source is None:
        opts.source = sources[0][0]
    
    if opts.register and opts.no_register:
        p.error('Conflicting options specified on whether to register '
                'setup install paths in configuration file.  Try not '
                'specifying some options.')

    # The sg15 file is a tarbomb.  We will later defuse it by untarring
    # into a subdirectory, so we don't leave a ghastly mess on the
    # unsuspecting user's system.
    
    if not opts.tarball:
        if opts.list_all:
            urls = []
            for source in names:
                urls1 = get_urls(source)
                urls.extend(urls1)
        else:
            urls = get_urls(opts.source)
    
        def print_urls(urls, marked=None):
            for url in urls:
                pageurl, fname = url.rsplit('/', 1)
                if url == marked:
                    marking = ' [*]'
                else:
                    marking = '    '
                print(' %s %s' % (marking, url))
    
        if len(urls) == 0:
            p.error(
                'For some reason, no files were found. Probably this script '
                'is out of date.  Please rummage around GPAW web page until '
                'solution is found.  Writing e-mails to '
                'gpaw-developers@lists@listserv.fysik.dtu.dk is also likely'
                'to help.')
    
        if opts.version:
            matching_urls = [url for url in urls if opts.version in url]
            if len(matching_urls) > 1:
                p.error('More than one setup file matches version "%s":\n'
                        '%s' % (opts.version, '\n'.join(matching_urls)))
            elif len(matching_urls) == 0:
                p.error('\nNo setup matched the specified version "%s".\n'
                        'Available setups are:\n'
                        '%s' % (opts.version, '\n'.join(urls)))
            assert len(matching_urls) == 1
            url = matching_urls[0]
        else:
            url = urls[0]
    
        print('Available setups and pseudopotentials')
        print_urls(urls, url)
        print()
        
    if nargs == 0:
        print_setups_info(p)
        print()
        progname = p.get_prog_name()
        print('Run %s DIR to install newest setups into DIR.' % progname)
        print('Run %s DIR --version=VERSION to install VERSION (from above).'
              % progname)
        print('See %s --help for more info.' % progname)
        raise SystemExit
    elif len(args) != 1:
        p.error('No more than one DIR expected.  Please try --help.')
    
    targetpath = args[0]
    
    if opts.tarball:
        print('Reading local tarball %s' % opts.tarball)
        targzfile = tarfile.open(opts.tarball)
        tarfname = opts.tarball
    else:
        tarfname = url.rsplit('/', 1)[1]
        print('Selected %s.  Downloading...' % tarfname)
        response = urllib2.urlopen(url)
        targzfile = tarfile.open(fileobj=StringIO(response.read()))
    
    if not os.path.exists(targetpath):
        os.makedirs(targetpath)
    
    assert tarfname.endswith('.tar.gz')
    setup_dirname = tarfname.rsplit('.', 2)[0]  # remove .tar.gz ending
    setup_path = os.path.abspath(os.path.join(targetpath, setup_dirname))
    if tarfname.startswith('sg15'):
        # Defuse tarbomb
        if not os.path.isdir(setup_path):
            os.mkdir(setup_path)
        targetpath = os.path.join(targetpath, setup_dirname)
    
    print('Extracting tarball into %s' % targetpath)
    targzfile.extractall(targetpath)
    assert os.path.isdir(setup_path)
    print('Setups installed into %s.' % setup_path)
    
    # Okay, now we have to maybe edit people's rc files.
    rcfiledir = os.path.join(os.environ['HOME'], '.gpaw')
    rcfilepath = os.path.join(rcfiledir, 'rc.py')
    
    # We could do all this by importing the rcfile as well and checking
    # whether things are okay or not.
    rcline = "setup_paths.insert(0, '%s')" % setup_path
    
    # Run interactive mode unless someone specified a flag requiring otherwise
    interactive_mode = not (opts.register or opts.no_register)
    
    register_path = False
    
    if interactive_mode:
        answer = raw_input('Register this setup path in %s? [y/n] ' %
                           rcfilepath)
        if answer.lower() in ['y', 'yes']:
            register_path = True
        elif answer.lower() in ['n', 'no']:
            print('As you wish.')
        else:
            print('What do you mean by "%s"?  Assuming "n".' % answer)
    else:
        if opts.register:
            assert not opts.no_register
            register_path = True
        else:
            assert opts.no_register
    
    if register_path:
        # First we create the file
        if not os.path.exists(rcfiledir):
            os.makedirs(rcfiledir)
        if not os.path.exists(rcfilepath):
            tmpfd = open(rcfilepath, 'w')  # Just create empty file
            tmpfd.close()
    
        for line in open(rcfilepath):
            if line.startswith(rcline):
                print('It looks like the path is already registered in %s.'
                      % rcfilepath)
                print('File will not be modified at this time.')
                break
        else:
            rcfd = open(rcfilepath, 'a')
            print(rcline, file=rcfd)
            print('Setup path registered in %s.' % rcfilepath)
            # Need to explicitly flush/close the file so print_setups_info
            # sees the change in rc.py
            rcfd.close()
    
            print_setups_info(p)
    else:
        print('You can manually register the setups by adding the')
        print('following line to %s:' % rcfilepath)
        print()
        print(rcline)
        print()
    print('Installation complete.')
        
        
def get_urls(source):
    if source == 'gpaw':
        page = 'https://wiki.fysik.dtu.dk/gpaw/_sources/setups/setups.txt'
        response = urllib2.urlopen(page)
        pattern = 'https://wiki.fysik.dtu.dk/gpaw-files/gpaw-setups-*.tar.gz'
        urls = [line.strip() for line in response
                if fnmatch.fnmatch(line.strip(), pattern)]
        return urls
    elif source == 'sg15':
        page = 'http://fpmd.ucdavis.edu/qso/potentials/sg15_oncv/'
        response = urllib2.urlopen(page)
        # Extract filename from:
        # <a href="...">sg15_oncv_upf_YYYY-MM-DD.tar.gz</a>
        pattern = r'>(sg15_oncv_upf_\d\d\d\d-\d\d-\d\d\.tar\.gz)</a>'
        files = re.compile(pattern).findall(response.read())
        files.sort(reverse=True)
        urls = [page + fname for fname in files]
        return urls
    elif source == 'basis':
        page = 'http://dcwww.camd.dtu.dk/~askhl/files/gpaw-lcao-basis-sets/'
        pattern = re.compile('>(gpaw-basis-.+?.tar.gz)</a>')
        response = urllib2.urlopen(page)
        files = sorted(pattern.findall(response.read()), reverse=True)
        return [page + fname for fname in files]
    elif source == 'test':
        return ['http://dcwww.camd.dtu.dk/~askhl/files/gpaw-test-source/'
                'gpaw-dist-test-source.tar.gz']
    else:
        raise ValueError('Unknown source: %s' % source)

        
def print_setups_info(p):
    try:
        import gpaw
    except ImportError as e:
        p.error('Cannot import \'gpaw\'.  GPAW does not appear to be '
                'installed. %s' % e)

    # GPAW may already have been imported, and the contents of the rc
    # file may have changed since then.  Thus, we re-import gpaw to be
    # sure that everything is as it should be.
    reload(gpaw)

    npaths = len(gpaw.setup_paths)
    if npaths == 0:
        print('GPAW currently has no setup search paths')
    else:
        print('Current GPAW setup paths in order of search priority:')
        for i, path in enumerate(gpaw.setup_paths):
            print('%4d. %s' % (i + 1, path))



from __future__ import print_function
from ase import Atoms
from gpaw import GPAW, PW
from gpaw.mpi import rank, size, serial_comm
from gpaw.xc.hybridg import HybridXC

a = 2.0
li = Atoms('Li', cell=(a, a, a), pbc=1)
for spinpol in [False, True]:
    for symm in [{}, 'off', {'time_reversal': False, 'point_group': False}]:
        if size == 8 and not spinpol and symm == {}:
            continue
        for qparallel in [False, True]:
            if rank == 0:
                print((spinpol, symm, qparallel))
            li.calc = GPAW(mode=PW(300),
                           kpts=(2, 3, 4),
                           spinpol=spinpol,
                           symmetry=symm,
                           parallel={'band': 1},
                           txt=None,
                           idiotproof=False)
            e = li.get_potential_energy()
            if qparallel:
                li.calc.write('li', mode='all')
                calc = GPAW('li', txt=None, communicator=serial_comm)
            else:
                calc = li.calc
            exx = HybridXC('EXX',
                           logfilename=None,
                           method='acdf')
            de = calc.get_xc_difference(exx)
            exx = HybridXC('EXX',
                           logfilename=None,
                           method='acdf',
                           bandstructure=True, bands=[0, 1])
            de2 = calc.get_xc_difference(exx)
            kd = calc.wfs.kd
            print(e, -0.56024, abs(e - -0.56024))
            print(de, -0.4520, abs(de - -0.4520))
            print(de, de2, abs(de - de2))
            assert abs(e - -0.56024) < 1e-5, abs(e)
            assert abs(de - -0.4520) < 3e-4, abs(de)
            assert abs(de - de2) < 1e-12
            for k in range(kd.nibzkpts):
                if abs(kd.ibzk_kc[k] - [0.25, 1 / 3.0, 3 / 8.0]).max() < 1e-7:
                    assert abs(exx.exx_skn[:, k, 0] - -0.18246).max() < 1e-5



from math import pi
from gpaw.grid_descriptor import GridDescriptor
from gpaw.xc import XC
from gpaw.xc.noncollinear import NonCollinearLDAKernel, NonCollinearFunctional
import numpy as np
from gpaw.test import equal

N = 20
a = 1.0
gd = GridDescriptor((N, N, N), (a, a, a))

for xc in [XC('LDA'), XC('PBE')]:
    n = gd.empty(2)
    n.fill(0.03)
    z = np.arange(gd.beg_c[2], gd.end_c[2]) * a / N
    n[:] += 0.01 * np.sin(2 * pi * z / a)
    n[1] += 0.02 + 0.01 * np.cos(2 * pi * z / a)
    n /= 2

    v = 0.0 * n
    E = xc.calculate(gd, n, v)

    here = (gd.beg_c[0] <= 1 < gd.end_c[0] and
            gd.beg_c[1] <= 2 < gd.end_c[1] and
            gd.beg_c[2] <= 3 < gd.end_c[2])
    if here:
        x = v[-1, 1, 2, 3] * gd.dv
        n[-1, 1, 2, 3] += 0.000001
    Ep = xc.calculate(gd, n, v)
    if here:
        n[-1, 1, 2, 3] -= 0.000002
    Em = xc.calculate(gd, n, v)
    x2 = (Ep - Em) / 0.000002
    if here:
        print(xc.name, E, x, x2, x - x2)
        equal(x, x2, 1e-11)
        n[-1, 1, 2, 3] += 0.000001

    if 0:#xc.type == 'LDA':
        xc = XC(NonCollinearLDAKernel())
    else:
        xc = NonCollinearFunctional(xc)

    n2 = gd.zeros(4)
    n2[0] = n.sum(0)
    n2[3] = n[0] - n[1]
    E2 = xc.calculate(gd, n2)
    print(E, E2-E)
    assert abs(E2 - E) < 1e-11
    n2[1] = 0.1 * n2[3]
    n2[2] = 0.2 * n2[3]
    n2[3] *= (1 - 0.1**2 - 0.2**2)**0.5
    v = n2 * 0
    E2 = xc.calculate(gd, n2, v)
    print(E, E2-E)
    assert abs(E2 - E) < 1e-11

    for i in range(4):
        if here:
            x = v[i, 1, 2, 3] * gd.dv
            n2[i, 1, 2, 3] += 0.000001
        Ep = xc.calculate(gd, n2)
        if here:
            n2[i, 1, 2, 3] -= 0.000002
        Em = xc.calculate(gd, n2)
        x2 = (Ep - Em) / 0.000002
        if here:
            print(i, x, x2, x - x2)
            equal(x, x2, 1e-11)
            n2[i, 1, 2, 3] += 0.000001



from gpaw.transport.tools import dagger, dot
from gpaw.transport.sparse_matrix import Banded_Sparse_Matrix
import copy
import numpy as np
import pickle


class LeadSelfEnergy:
    # This object use the sparse_matrix object Banded_Sparse_HSD
    conv = 1e-8  # Convergence criteria for surface Green function
    
    def __init__(self, hsd_ii, hsd_ij, data_path=None, direction='left',
                 rotation_mat=None):
        self.hsd_ii = hsd_ii
        self.hsd_ij = hsd_ij
        self.data_path = data_path
        self.direction = direction
        self.energy = None
        self.rotation_mat = rotation_mat
        self.bias = 0
        self.s = 0
        self.pk = 0
        self.nid_plus = 30
        self.sigma = None
        self.tol = 1e-6
        
    def __call__(self, energy, flag=None):
        if self.data_path is not None and flag is not None:
            nid = int(flag[2:])
            nid += self.nid_plus
            flag = str(flag[:2]) + str(nid)
            fd = open(self.data_path[self.direction] + '/' +
                      self.direction + '/' + flag, 'rb')
            data = pickle.load(fd)
            return data
        else:
            if self.energy is None or abs(energy - self.energy) > self.tol:
                self.energy = energy
                z = energy - self.bias
                tau_ij = z * self.hsd_ij.S[self.pk].recover() - \
                                     self.hsd_ij.H[self.s][self.pk].recover()
                tau_ji = z * dagger(self.hsd_ij.S[self.pk].recover()) - \
                             dagger(self.hsd_ij.H[self.s][self.pk].recover())
                ginv = self.get_sgfinv(energy)
                a_ij = dot(ginv, tau_ij)
                mat = dot(tau_ji, a_ij)
                #if self.rotation_mat is not None:
                #    mat = np.dot(self.rotation_mat, mat)
                #    mat = np.dot(mat, self.rotation_mat.T)
                self.sigma = Banded_Sparse_Matrix(complex, mat,
                                            self.hsd_ii.S[self.pk].band_index)
            return self.sigma
       
    def set_bias(self, bias):
        self.bias = bias
        self.nid_plus = -int(bias // 0.05) + 30
        
    def get_lambda(self, energy):
        sigma = self(energy)
        sigma_mm = sigma.recover()
        return 1.j * (sigma_mm - dagger(sigma_mm))
    
    def get_sgfinv(self, energy):
        """The inverse of the retarded surface Green function"""
        z = energy - self.bias
        v_00 = Banded_Sparse_Matrix(complex,
                                     None,
                                     self.hsd_ii.S[self.pk].band_index)
        
        v_00.reset_from_others(self.hsd_ii.S[self.pk],
                                self.hsd_ii.H[self.s][self.pk],
                                z, -1.0)
        
        v_11 = copy.deepcopy(v_00)
        
        v_10 = z * self.hsd_ij.S[self.pk].recover()- \
                                     self.hsd_ij.H[self.s][self.pk].recover()
        v_01 = z * dagger(self.hsd_ij.S[self.pk].recover()) - \
                              dagger(self.hsd_ij.H[self.s][self.pk].recover())
        delta = self.conv + 1
        while delta > self.conv:
            inv_v_11 = v_11.inv()
            a = dot(inv_v_11, v_01)
            b = dot(inv_v_11, v_10)
            v_01_dot_b = dot(v_01, b)
            v_00.reset_minus(v_01_dot_b, full=True)
            v_11.reset_minus(dot(v_10, a), full=True)
            v_11.reset_minus(v_01_dot_b, full=True)
            v_01 = -dot(v_01, a)
            v_10 = -dot(v_10, b)
            delta = np.abs(v_01).max()
        return v_00.inv()





# Copyright (C) 2010  CAMd
# Copyright (C) 2010  Argonne National Laboratory
# Please see the accompanying LICENSE file for further information.
import numpy as np

from gpaw.matrix_descriptor import MatrixDescriptor, \
    BandMatrixDescriptor, \
    BlacsBandMatrixDescriptor
from gpaw.blacs import BlacsGrid, Redistributor
from gpaw.utilities import uncamelcase
from gpaw.utilities.blas import gemm, r2k
from gpaw.utilities.lapack import diagonalize, general_diagonalize, \
    inverse_cholesky
from gpaw.utilities.scalapack import pblas_simple_gemm, pblas_tran
from gpaw.utilities.tools import tri2full
from gpaw.utilities.timing import nulltimer


def get_KohnSham_layouts(sl, mode, gd, bd, block_comm, dtype, **kwargs):
    """Create Kohn-Sham layouts object."""
    # Not needed for AtomPAW special mode, as usual we just provide whatever
    # happens to make the code not crash
    if not isinstance(mode, str):
        return None  #XXX
    name = {'fd': 'BandLayouts', 'lcao': 'OrbitalLayouts'}[mode]
    args = (gd, bd, block_comm, dtype)
    if sl is not None:
        name = 'Blacs' + name
        assert len(sl) == 3
        args += tuple(sl)
    ksl = {'BandLayouts':         BandLayouts,
           'BlacsBandLayouts':    BlacsBandLayouts,
           'BlacsOrbitalLayouts': BlacsOrbitalLayouts,
           'OrbitalLayouts':      OrbitalLayouts,
            }[name](*args, **kwargs)
    if 0: #XXX debug
        print(('USING KSL: %s' % repr(ksl)))
    assert isinstance(ksl, KohnShamLayouts)
    assert isinstance(ksl, BlacsLayouts) == (sl is not None)
    return ksl


class KohnShamLayouts:
    using_blacs = False  # This is only used by a regression test
    matrix_descriptor_class = None

    def __init__(self, gd, bd, block_comm, dtype, timer=nulltimer):
        assert gd.comm.parent is bd.comm.parent  # must have same parent comm
        self.world = bd.comm.parent
        self.gd = gd
        self.bd = bd
        self.dtype = dtype
        self.block_comm = block_comm
        self.timer = timer
        self._kwargs = {'timer': timer}

        if gd.comm.rank == 0:
            self.column_comm = bd.comm
        else:
            self.column_comm = None

    def get_keywords(self):
        return self._kwargs.copy()  # just a shallow copy...

    def diagonalize(self, *args, **kwargs):
        raise RuntimeError('Virtual member function should not be called.')

    def inverse_cholesky(self, *args, **kwargs):
        raise RuntimeError('Virtual member function should not be called.')

    def new_descriptor(self):
        return self.matrix_descriptor_class(self.bd, self.gd, self)

    def __repr__(self):
        return uncamelcase(self.__class__.__name__)

    def get_description(self):
        """Description of this object in prose, e.g. for logging.

        Subclasses are expected to override this with something useful."""
        return repr(self)


class BlacsLayouts(KohnShamLayouts):
    using_blacs = True  # This is only used by a regression test

    def __init__(self, gd, bd, block_comm, dtype, mcpus, ncpus,
                 blocksize, timer=nulltimer):
        KohnShamLayouts.__init__(self, gd, bd, block_comm, dtype,
                                 timer)
        # WARNING: Do not create the BlacsGrid on a communicator which does not
        # contain block_comm.rank = 0. This will break BlacsBandLayouts which
        # assume eps_M will be broadcast over block_comm.
        self.blocksize = blocksize
        self.blockgrid = BlacsGrid(self.block_comm, mcpus, ncpus)

    def get_description(self):
        title = 'BLACS'
        template = '%d x %d grid with %d x %d blocksize'
        return (title, template)


class BandLayouts(KohnShamLayouts):
    matrix_descriptor_class = BandMatrixDescriptor

    def __init__(self, gd, bd, block_comm, dtype,
                 buffer_size=None, timer=nulltimer):
        KohnShamLayouts.__init__(self, gd, bd, block_comm, dtype,
                                 timer)
        self.buffer_size = buffer_size

    def diagonalize(self, H_NN, eps_n):
        """Serial diagonalizer must handle two cases:
        1. Parallelization over domains only.
        2. Simultaneous parallelization over domains and bands.
        """
        nbands = self.bd.nbands
        eps_N = np.empty(nbands)
        self.timer.start('Diagonalize')
        # Broadcast on block_comm since result
        # is k-point and spin-dependent only
        self.block_comm.broadcast(H_NN, 0)
        # The result on different processor is not necessarily bit-wise
        # identical, so only domain master performs diagonalization
        if self.gd.comm.rank == 0:
            self._diagonalize(H_NN, eps_N)
        self.gd.comm.broadcast(H_NN, 0)
        self.gd.comm.broadcast(eps_N, 0)
        self.timer.stop('Diagonalize')

        self.timer.start('Distribute results')
        # Copy the portion that belongs to my band group
        eps_n[:] = eps_N[self.bd.get_slice()]
        self.timer.stop('Distribute results')

    def _diagonalize(self, H_NN, eps_N):
        """Serial diagonalize via LAPACK."""
        # This is replicated computation but ultimately avoids
        # additional communication.
        diagonalize(H_NN, eps_N)

    def inverse_cholesky(self, S_NN):
        """Serial inverse Cholesky must handle two cases:
        1. Parallelization over domains only.
        2. Simultaneous parallelization over domains and bands.
        """
        self.timer.start('Inverse Cholesky')
        # Broadcast on block_comm since result
        # is k-point and spin-dependent only
        self.block_comm.broadcast(S_NN, 0)
        # The result on different processor is not necessarily bit-wise
        # identical, so only domain master performs computation
        if self.gd.comm.rank == 0:
            self._inverse_cholesky(S_NN)
        self.gd.comm.broadcast(S_NN, 0)
        self.timer.stop('Inverse Cholesky')

    def _inverse_cholesky(self, S_NN):
        """Serial inverse Cholesky via LAPACK."""
        # This is replicated computation but ultimately avoids
        # additional communication.
        inverse_cholesky(S_NN)

    def get_description(self):
        return 'Serial LAPACK'


class BlacsBandLayouts(BlacsLayouts):  #XXX should derive from BandLayouts too!
    """ScaLAPACK Dense Linear Algebra.

    This class is instantiated in the real-space code.  Not for
    casual use, at least for now.
    
    Requires two distributors and three descriptors for initialization
    as well as grid descriptors and band descriptors. Distributors are
    for cols2blocks (1D -> 2D BLACS grid) and blocks2rows (2D -> 1D
    BLACS grid). ScaLAPACK operations must occur on a 2D BLACS grid for
    performance and scalability. Redistribute of 1D *column* layout
    matrix will operate only on lower half of H or S. Redistribute of
    2D block will operate on entire matrix for U, but only lower half
    of C.

    inverse_cholesky is "hard-coded" for real-space code.
    Expects overlap matrix (S) and the coefficient matrix (C) to be a
    replicated data structures and *not* created by the BLACS descriptor class.
    This is due to the MPI_Reduce and MPI_Broadcast that will occur
    in the parallel matrix multiply. Input matrices should be:
    S = np.empty((nbands, mybands), dtype)
    C = np.empty((mybands, nbands), dtype)

    
    _standard_diagonalize is "hard-coded" for the real-space code.
    Expects both hamiltonian (H) and eigenvector matrix (U) to be a
    replicated data structures and not created by the BLACS descriptor class.
    This is due to the MPI_Reduce and MPI_Broadcast that will occur
    in the parallel matrix multiply. Input matrices should be:
    H = np.empty((nbands, mynbands), dtype)
    U = np.empty((mynbands, nbands), dtype)
    eps_n = np.empty(mynbands, dtype = float)
    """ #XXX rewrite this docstring a bit!

    matrix_descriptor_class = BlacsBandMatrixDescriptor

    # This class 'describes' all the realspace Blacs-related layouts
    def __init__(self, gd, bd, block_comm, dtype, mcpus, ncpus,
                 blocksize, buffer_size=None, timer=nulltimer):
        BlacsLayouts.__init__(self, gd, bd, block_comm, dtype,
                              mcpus, ncpus, blocksize, timer)
        self.buffer_size = buffer_size
        nbands = bd.nbands
        self.mynbands = mynbands = bd.mynbands
        self.blocksize = blocksize

        # 1D layout - columns
        self.columngrid = BlacsGrid(self.column_comm, 1, bd.comm.size)
        self.Nndescriptor = self.columngrid.new_descriptor(nbands, nbands,
                                                           nbands, mynbands)

        # 2D layout
        self.nndescriptor = self.blockgrid.new_descriptor(nbands, nbands,
                                                          blocksize, blocksize)

        # 1D layout - rows
        self.rowgrid = BlacsGrid(self.column_comm, bd.comm.size, 1)
        self.nNdescriptor = self.rowgrid.new_descriptor(nbands, nbands,
                                                        mynbands, nbands)

        # Only redistribute filled out half for Hermitian matrices
        self.Nn2nn = Redistributor(self.block_comm, self.Nndescriptor,
                                   self.nndescriptor)
        #self.Nn2nn = Redistributor(self.block_comm, self.Nndescriptor,
        #                           self.nndescriptor, 'L') #XXX faster but...

        # Resulting matrix will be used in dgemm which is symmetry obvlious
        self.nn2nN = Redistributor(self.block_comm, self.nndescriptor,
                                   self.nNdescriptor)
        
    def diagonalize(self, H_nn, eps_n):
        nbands = self.bd.nbands
        eps_N = np.empty(nbands)
        self.timer.start('Diagonalize')
        self._diagonalize(H_nn, eps_N)
        self.timer.stop('Diagonalize')

        self.timer.start('Distribute results')
        # eps_N is already on block_comm.rank = 0
        # easier to broadcast eps_N to all and
        # get the correct slice afterward.
        self.block_comm.broadcast(eps_N, 0)
        eps_n[:] = eps_N[self.bd.get_slice()]
        self.timer.stop('Distribute results')

    def _diagonalize(self, H_nn, eps_N):
        """Parallel diagonalizer."""
        self.nndescriptor.diagonalize_dc(H_nn.copy(), H_nn, eps_N, 'L')
        
    def inverse_cholesky(self, S_nn):
        self.timer.start('Inverse Cholesky')
        self._inverse_cholesky(S_nn)
        self.block_comm.barrier() # removing barrier may lead to race condition
        self.timer.stop('Inverse Cholesky')
        
    def _inverse_cholesky(self, S_nn):
        self.nndescriptor.inverse_cholesky(S_nn, 'L')

    def get_description(self):
        (title, template) = BlacsLayouts.get_description(self)
        bg = self.blockgrid
        desc = self.nndescriptor
        s = template % (bg.nprow, bg.npcol, desc.mb, desc.nb)
        return ' '.join([title, s])


class BlacsOrbitalLayouts(BlacsLayouts):
    """ScaLAPACK Dense Linear Algebra.

    This class is instantiated in LCAO.  Not for casual use, at least for now.
    
    Requires two distributors and three descriptors for initialization
    as well as grid descriptors and band descriptors. Distributors are
    for cols2blocks (1D -> 2D BLACS grid) and blocks2cols (2D -> 1D
    BLACS grid). ScaLAPACK operations must occur on 2D BLACS grid for
    performance and scalability.

    _general_diagonalize is "hard-coded" for LCAO.
    Expects both Hamiltonian and Overlap matrix to be on the 2D BLACS grid.
    This is done early on to save memory.
    """
    # XXX rewrite this docstring a bit!

    # This class 'describes' all the LCAO Blacs-related layouts
    def __init__(self, gd, bd, block_comm, dtype, mcpus, ncpus,
                 blocksize, nao, timer=nulltimer):
        BlacsLayouts.__init__(self, gd, bd, block_comm, dtype,
                              mcpus, ncpus, blocksize, timer)
        nbands = bd.nbands
        self.blocksize = blocksize
        self.mynbands = mynbands = bd.mynbands
        
        self.orbital_comm = self.bd.comm
        self.naoblocksize = naoblocksize = -((-nao) // self.orbital_comm.size)
        self.nao = nao

        # Range of basis functions for BLACS distribution of matrices:
        self.Mmax = nao
        self.Mstart = bd.comm.rank * naoblocksize
        self.Mstop = min(self.Mstart + naoblocksize, self.Mmax)
        self.mynao = self.Mstop - self.Mstart

        # Column layout for one matrix per band rank:
        self.columngrid = BlacsGrid(bd.comm, bd.comm.size, 1)
        self.mMdescriptor = self.columngrid.new_descriptor(nao, nao,
                                                           naoblocksize, nao)
        self.nMdescriptor = self.columngrid.new_descriptor(nbands, nao,
                                                           mynbands, nao)

        #parallelprint(world, (mynao, self.mMdescriptor.shape))

        # Column layout for one matrix in total (only on grid masters):
        self.single_column_grid = BlacsGrid(self.column_comm, bd.comm.size, 1)
        self.mM_unique_descriptor = self.single_column_grid.new_descriptor( \
            nao, nao, naoblocksize, nao)

        # nM_unique_descriptor is meant to hold the coefficients after
        # diagonalization.  BLACS requires it to be nao-by-nao, but
        # we only fill meaningful data into the first nbands columns.
        #
        # The array will then be trimmed and broadcast across
        # the grid descriptor's communicator.
        self.nM_unique_descriptor = self.single_column_grid.new_descriptor( \
            nbands, nao, mynbands, nao)

        # Fully blocked grid for diagonalization with many CPUs:
        self.mmdescriptor = self.blockgrid.new_descriptor(nao, nao, blocksize,
                                                          blocksize)

        #self.nMdescriptor = nMdescriptor
        self.mM2mm = Redistributor(self.block_comm, self.mM_unique_descriptor,
                                   self.mmdescriptor)
        self.mm2nM = Redistributor(self.block_comm, self.mmdescriptor,
                                   self.nM_unique_descriptor)

    def diagonalize(self, H_mm, C_nM, eps_n, S_mm):
        # C_nM needs to be simultaneously compatible with:
        # 1. outdescriptor
        # 2. broadcast with gd.comm
        # We will does this with a dummy buffer C2_nM
        outdescriptor = self.mm2nM.dstdescriptor  # blocks2cols
        blockdescriptor = self.mM2mm.dstdescriptor  # cols2blocks

        dtype = S_mm.dtype
        eps_M = np.empty(C_nM.shape[-1])  # empty helps us debug
        subM, subN = outdescriptor.gshape
        
        C_mm = blockdescriptor.zeros(dtype=dtype)
        self.timer.start('General diagonalize')
        # general_diagonalize_ex may have a buffer overflow, so
        # we no longer use it
        #blockdescriptor.general_diagonalize_ex(H_mm, S_mm.copy(), C_mm, eps_M,
        #                                       UL='L', iu=self.bd.nbands)
        blockdescriptor.general_diagonalize_dc(H_mm, S_mm.copy(), C_mm, eps_M,
                                               UL='L')
        self.timer.stop('General diagonalize')
 
        # Make C_nM compatible with the redistributor
        self.timer.start('Redistribute coefs')
        if outdescriptor:
            C2_nM = C_nM
        else:
            C2_nM = outdescriptor.empty(dtype=dtype)
        assert outdescriptor.check(C2_nM)
        self.mm2nM.redistribute(C_mm, C2_nM, subM, subN)  # blocks2cols
        self.timer.stop('Redistribute coefs')

        self.timer.start('Send coefs to domains')
        # eps_M is already on block_comm.rank = 0
        # easier to broadcast eps_M to all and
        # get the correct slice afterward.
        self.block_comm.broadcast(eps_M, 0)
        eps_n[:] = eps_M[self.bd.get_slice()]
        self.gd.comm.broadcast(C_nM, 0)
        self.timer.stop('Send coefs to domains')

    def distribute_overlap_matrix(self, S_qmM, root=0,
                                  add_hermitian_conjugate=False):
        # Some MPI implementations need a lot of memory to do large
        # reductions.  To avoid trouble, we do comm.sum on smaller blocks
        # of S (this code is also safe for arrays smaller than blocksize)
        Sflat_x = S_qmM.ravel()
        blocksize = 2**23 // Sflat_x.itemsize  # 8 MiB
        nblocks = -(-len(Sflat_x) // blocksize)
        Mstart = 0
        self.timer.start('blocked summation')
        for i in range(nblocks):
            self.gd.comm.sum(Sflat_x[Mstart:Mstart + blocksize], root=root)
            Mstart += blocksize
        assert Mstart + blocksize >= len(Sflat_x)
        self.timer.stop('blocked summation')

        xshape = S_qmM.shape[:-2]
        nm, nM = S_qmM.shape[-2:]
        S_qmM = S_qmM.reshape(-1, nm, nM)
        
        blockdesc = self.mmdescriptor
        coldesc = self.mM_unique_descriptor
        S_qmm = blockdesc.zeros(len(S_qmM), S_qmM.dtype)

        if not coldesc:  # XXX ugly way to sort out inactive ranks
            S_qmM = coldesc.zeros(len(S_qmM), S_qmM.dtype)
        
        self.timer.start('Scalapack redistribute')
        for S_mM, S_mm in zip(S_qmM, S_qmm):
            self.mM2mm.redistribute(S_mM, S_mm)
            if add_hermitian_conjugate:
                if blockdesc.active:
                    pblas_tran(1.0, S_mm.copy(), 1.0, S_mm,
                               blockdesc, blockdesc)
                
        self.timer.stop('Scalapack redistribute')
        return S_qmm.reshape(xshape + blockdesc.shape)

    def get_overlap_matrix_shape(self):
        return self.mmdescriptor.shape

    def calculate_blocked_density_matrix(self, f_n, C_nM):
        nbands = self.bd.nbands
        nao = self.nao
        dtype = C_nM.dtype
        
        self.nMdescriptor.checkassert(C_nM)
        if self.gd.rank == 0:
            Cf_nM = (C_nM * f_n[:, None]).conj()
        else:
            C_nM = self.nM_unique_descriptor.zeros(dtype=dtype)
            Cf_nM = self.nM_unique_descriptor.zeros(dtype=dtype)

        r = Redistributor(self.block_comm, self.nM_unique_descriptor,
                          self.mmdescriptor)

        Cf_mm = self.mmdescriptor.zeros(dtype=dtype)
        r.redistribute(Cf_nM, Cf_mm, nbands, nao)
        del Cf_nM
        
        C_mm = self.mmdescriptor.zeros(dtype=dtype)
        r.redistribute(C_nM, C_mm, nbands, nao)
        # no use to delete C_nM as it's in the input...

        rho_mm = self.mmdescriptor.zeros(dtype=dtype)
        
        pblas_simple_gemm(self.mmdescriptor,
                          self.mmdescriptor,
                          self.mmdescriptor,
                          Cf_mm, C_mm, rho_mm, transa='T')
        return rho_mm

    def calculate_density_matrix(self, f_n, C_nM, rho_mM=None):
        """Calculate density matrix from occupations and coefficients.

        Presently this function performs the usual scalapack 3-step trick:
        redistribute-numbercrunching-backdistribute.
        
        
        Notes on future performance improvement.
        
        As per the current framework, C_nM exists as copies on each
        domain, i.e. this is not parallel over domains.  We'd like to
        correct this and have an efficient distribution using e.g. the
        block communicator.

        The diagonalization routine and other parts of the code should
        however be changed to accommodate the following scheme:
        
        Keep coefficients in C_mm form after the diagonalization.
        rho_mm can then be directly calculated from C_mm without
        redistribution, after which we only need to redistribute
        rho_mm across domains.
        
        """
        dtype = C_nM.dtype
        rho_mm = self.calculate_blocked_density_matrix(f_n, C_nM)
        rback = Redistributor(self.block_comm, self.mmdescriptor,
                              self.mM_unique_descriptor)
        rho1_mM = self.mM_unique_descriptor.zeros(dtype=dtype)
        rback.redistribute(rho_mm, rho1_mM)
        del rho_mm

        if rho_mM is None:
            if self.gd.rank == 0:
                rho_mM = rho1_mM
            else:
                rho_mM = self.mMdescriptor.zeros(dtype=dtype)

        self.gd.comm.broadcast(rho_mM, 0)
        return rho_mM

    def distribute_to_columns(self, rho_mm, srcdescriptor):
        redistributor = Redistributor(self.block_comm, # XXX
                                      srcdescriptor,
                                      self.mM_unique_descriptor)
        rho_mM = redistributor.redistribute(rho_mm)
        if self.gd.rank != 0:
            rho_mM = self.mMdescriptor.zeros(dtype=rho_mm.dtype)
        self.gd.comm.broadcast(rho_mM, 0)
        return rho_mM

    def oldcalculate_density_matrix(self, f_n, C_nM, rho_mM=None):
        # This version is parallel over the band descriptor only.
        # This is inefficient, but let's keep it for a while in case
        # there's trouble with the more efficient version
        if rho_mM is None:
            rho_mM = self.mMdescriptor.zeros(dtype=C_nM.dtype)
        
        Cf_nM = (C_nM * f_n[:, None]).conj()
        pblas_simple_gemm(self.nMdescriptor, self.nMdescriptor,
                          self.mMdescriptor, Cf_nM, C_nM, rho_mM, transa='T')
        return rho_mM

    def get_transposed_density_matrix(self, f_n, C_nM, rho_mM=None):
        return self.calculate_density_matrix(f_n, C_nM, rho_mM).conj()

    def get_description(self):
        (title, template) = BlacsLayouts.get_description(self)
        bg = self.blockgrid
        desc = self.mmdescriptor
        s = template % (bg.nprow, bg.npcol, desc.mb, desc.nb)
        return ' '.join([title, s])


class OrbitalLayouts(KohnShamLayouts):
    def __init__(self, gd, bd, block_comm, dtype, nao,
                 timer=nulltimer):
        KohnShamLayouts.__init__(self, gd, bd, block_comm, dtype,
                                 timer)
        self.mMdescriptor = MatrixDescriptor(nao, nao)
        self.nMdescriptor = MatrixDescriptor(bd.mynbands, nao)
        
        self.Mstart = 0
        self.Mstop = nao
        self.Mmax = nao
        self.mynao = nao
        self.nao = nao
        self.orbital_comm = bd.comm

    def diagonalize(self, H_MM, C_nM, eps_n, S_MM):
        eps_M = np.empty(C_nM.shape[-1])
        self.block_comm.broadcast(H_MM, 0)
        self.block_comm.broadcast(S_MM, 0)
        # The result on different processor is not necessarily bit-wise
        # identical, so only domain master performs computation
        if self.gd.comm.rank == 0:
            self._diagonalize(H_MM, S_MM.copy(), eps_M)
        self.gd.comm.broadcast(H_MM, 0)
        self.gd.comm.broadcast(eps_M, 0)
        eps_n[:] = eps_M[self.bd.get_slice()]
        C_nM[:] = H_MM[self.bd.get_slice()]
    
    def _diagonalize(self, H_MM, S_MM, eps_M):
        """Serial diagonalize via LAPACK."""
        # This is replicated computation but ultimately avoids
        # additional communication
        general_diagonalize(H_MM, eps_M, S_MM)

    def estimate_memory(self, mem, dtype):
        itemsize = mem.itemsize[dtype]
        mem.subnode('eps [M]', self.nao * mem.floatsize)
        mem.subnode('H [MM]', self.nao * self.nao * itemsize)

    def distribute_overlap_matrix(self, S_qMM, root=0,
                                  add_hermitian_conjugate=False):
        self.gd.comm.sum(S_qMM, root)
        if add_hermitian_conjugate:
            S_qMM += S_qMM.swapaxes(-1, -2).conj()
        return S_qMM

    def get_overlap_matrix_shape(self):
        return self.nao, self.nao

    def calculate_density_matrix(self, f_n, C_nM, rho_MM=None, C2_nM=None):
        # Only a madman would use a non-transposed density matrix.
        # Maybe we should use the get_transposed_density_matrix instead
        if rho_MM is None:
            rho_MM = np.zeros((self.mynao, self.nao), dtype=C_nM.dtype)
        # XXX Should not conjugate, but call gemm(..., 'c')
        # Although that requires knowing C_Mn and not C_nM.
        # that also conforms better to the usual conventions in literature
        if C2_nM is None:
            C2_nM = C_nM
        Cf_Mn = np.ascontiguousarray(C2_nM.T.conj() * f_n)
        gemm(1.0, C_nM, Cf_Mn, 0.0, rho_MM, 'n')
        return rho_MM

    def get_transposed_density_matrix(self, f_n, C_nM, rho_MM=None):
        return self.calculate_density_matrix(f_n, C_nM, rho_MM).T.copy()

        #if rho_MM is None:
        #    rho_MM = np.zeros((self.mynao, self.nao), dtype=C_nM.dtype)
        #C_Mn = C_nM.T.copy()
        #gemm(1.0, C_Mn, f_n[np.newaxis, :] * C_Mn, 0.0, rho_MM, 'c')
        #self.bd.comm.sum(rho_MM)
        #return rho_MM

    def alternative_calculate_density_matrix(self, f_n, C_nM, rho_MM=None):
        if rho_MM is None:
            rho_MM = np.zeros((self.mynao, self.nao), dtype=C_nM.dtype)
        # Alternative suggestion. Might be faster. Someone should test this
        C_Mn = C_nM.T.copy()
        r2k(0.5, C_Mn, f_n * C_Mn, 0.0, rho_MM)
        tri2full(rho_MM)
        return rho_MM

    def get_description(self):
        return 'Serial LAPACK'

    def calculate_density_matrix_delta(self, d_nn, C_nM, rho_MM=None):
        # Only a madman would use a non-transposed density matrix.
        # Maybe we should use the get_transposed_density_matrix instead
        if rho_MM is None:
            rho_MM = np.zeros((self.mynao, self.nao), dtype=C_nM.dtype)
        Cd_Mn = np.zeros((self.nao, self.bd.mynbands), dtype=C_nM.dtype)
        # XXX Should not conjugate, but call gemm(..., 'c')
        # Although that requires knowing C_Mn and not C_nM.
        # that also conforms better to the usual conventions in literature
        C_Mn = C_nM.T.conj().copy()
        gemm(1.0, d_nn, C_Mn, 0.0, Cd_Mn, 'n')
        gemm(1.0, C_nM, Cd_Mn, 0.0, rho_MM, 'n')
        self.bd.comm.sum(rho_MM)
        return rho_MM

    def get_transposed_density_matrix_delta(self, d_nn, C_nM, rho_MM=None):
        return self.calculate_density_matrix_delta(d_nn, C_nM, rho_MM).T.copy()



# gpaw-python Segmentation faults
# when gpaw-python and numpy are linked to different blas

import numpy as np
import sys

from math import sqrt
from ase import Atom, Atoms
from gpaw import GPAW
from gpaw import ConvergenceError

kpts = (2,1,1)
a=1.42
c=3.355

# AB stack
atoms = Atoms('C4',[
              (1/3.0,1/3.0,0),
              (2/3.0,2/3.0,0),
              (0.   ,0.   ,0.5),
              (1/3.0,1/3.0,0.5)
              ],
              pbc=(1,1,1))
atoms.set_cell([(sqrt(3)*a/2.0,3/2.0*a,0),
                (-sqrt(3)*a/2.0,3/2.0*a,0),
                (0.,0.,2*c)],
               scale_atoms=True)

calc = GPAW(gpts=(8, 8, 20), nbands=9, kpts=kpts, maxiter=1)

atoms.set_calculator(calc)

try:
    pot = atoms.get_potential_energy()
except ConvergenceError:
    pass



import sys
from ase import Atoms, Atom
from ase.vibrations.resonant_raman import ResonantRaman

from gpaw import GPAW, FermiDirac
from gpaw.cluster import Cluster
from gpaw.lrtddft.kssingle import KSSingles
from gpaw.mpi import world

txt = '-'
txt = None
load = True
load = False
xc = 'LDA'

R = 0.7  # approx. experimental bond length
a = 4.0
c = 5.0
H2 = Atoms([Atom('H', (a / 2, a / 2, (c - R) / 2)),
            Atom('H', (a / 2, a / 2, (c + R) / 2))],
           cell=(a, a, c))

calc = GPAW(xc=xc, nbands=3, spinpol=False, eigensolver='rmm-diis', txt=txt)
H2.set_calculator(calc)
H2.get_potential_energy()

gsname = exname = 'rraman'
rr = ResonantRaman(H2, KSSingles, gsname=gsname, exname=exname,
                   exkwargs={'eps':0.0, 'jend':1}
               )
rr.run()

# check size
kss = KSSingles('rraman-d0.010.eq.ex.gz')
assert(len(kss) == 1)

rr = ResonantRaman(H2, KSSingles, gsname=gsname, exname=exname, 
                   verbose=True,)
rr.summary(omega=5, method='frederiksen')
